#!/usr/bin/env python

# (c) Copyright 2011-2012 Cloudera, Inc.

import os
import logging
import stats

import clusterstats.common.configuration as configuration
import clusterstats.common.loggerconfig as loggerconfig

from fs_daos import MachineCollectionFsDao

config = configuration.initialize_config("cluster_stats", __file__)
logger = loggerconfig.get_and_configure_root_logger(config)
logger.setLevel(logging.INFO)

def run_sys_commands(machine_collection_dao, processors=None, timeout=None):
    """ Runs commands and puts the stdout and stderr in a new file in output_path. """
    runner = stats.create_sys_stat_runner(machine_collection_dao, processors, timeout)
    runner.run()
    machine_collection_dao.put_sys_stats_run(runner.get_run_order())

def collect_sys_stats(output_dir, processors=None, timeout=None):
    if not output_dir:
        raise Exception("Must supply an output path.")

    # setup logger handler to persist logs
    file_handler = logging.FileHandler(os.path.join(output_dir, "run.log"))
    file_handler.setFormatter(logging.Formatter(
            "%(asctime)s %(module)-25s %(levelname)-8s: %(message)s"))
    logger.addHandler(file_handler)
    
    machine_collection_dao = MachineCollectionFsDao(output_dir)
    logger.info("Writing files to %s" % output_dir)
    machine_collection_dao.initialize_performance()
    
    # always collect the system statistics
    run_sys_commands(machine_collection_dao, processors, timeout)
    machine_collection_dao.complete_performance()

if __name__ == "__main__":
    import sys
    if len(sys.argv) < 2:
        raise Exception("Usage: machine_stats.py OUTPUT_DIR")
    collect_sys_stats(sys.argv[1])
