# (c) Copyright 2010 Cloudera, Inc.

"""This module contains method chaining functionality."""

import inspect

class ChainError(Exception):
    pass

class Chain(object):
    """A chain of commands to call. The chain is itself callable like any command, so
    one could also chain chains together."""

    def __init__(self, commands):
        self.__commands = commands

    def __call__(self, context):
        """Calls all commands in its command list. If the command returns True, continues
        onto the next command until all commands have been called. If a command returns false,
        it exits the loop and returns immediately."""
        for command in self.__commands:
            succ = command(context)
            if not succ:
                return False
        return True

def build_chain(commands):
    """Build a chain that is itself callable. This construction function will also
    verify that commands are callable."""
    for command in commands:
        try:
            if not callable(command):
                raise AttributeError
        except AttributeError, err:
            msg = "Commands in chain must be callable, and must accept 1 arg."
            raise ChainError, msg
    return Chain(commands)
        
