# (c) Copyright 2011 Cloudera, Inc.

"""This module contains file filters that can be chained together to
filter out files in a context according to filter specific rules."""

class LogLevelFilter(object):
    """Filter out all log events with level below the level passed to the
    constructor."""

    levels = ["TRACE", "DEBUG", "INFO", "WARN", "ERROR", "FATAL"]

    def __init__(self, level):
        try:
            self.__level_index = LogLevelFilter.levels.index(level)
        except ValueError:
            raise ValueError("%s is not a valid log level" % (level,))

    def __call__(self, log_event):
        level = log_event.get_log_level()
        try:
            if self.__level_index <= LogLevelFilter.levels.index(level):
                return True
            else:
                return False
        except:
            return False

class HueLogLevelFilter(object):
    """Filter out all log events with level below the level passed to the
    constructor."""

    levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]

    def __init__(self, level):
        try:
            self.__level_index = HueLogLevelFilter.levels.index(level)
        except ValueError:
            raise ValueError("%s is not a valid log level" % (level,))

    def __call__(self, log_event):
        level = log_event.get_log_level()
        try:
            if self.__level_index <= HueLogLevelFilter.levels.index(level):
                return True
            else:
                return False
        except:
            return False

class GLogLogLevelFilter(object):
    """Filter out all log events with level below the level passed to the
    constructor."""

    # Log levels that Impala can have. As Impala uses two logging systems,
    # GLog and Log4J we have union of both log levels in the list
    levels = ["TRACE", "DEBUG", "INFO", "I", "WARN", "W", "ERROR", "E", "FATAL", "F"]

    def __init__(self, level):
        try:
            self.__level_index = GLogLogLevelFilter.levels.index(level)
        except ValueError:
            raise ValueError("%s is not a valid log level" % (level,))

    def __call__(self, log_event):
        level = log_event.get_log_level()
        try:
            if self.__level_index <= GLogLogLevelFilter.levels.index(level):
                return True
            else:
                return False
        except:
            return False
