# Copyright (c) 2014 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

def parse_generic_audit_event(service_type, service_name, event_from_audit_log,
    log_parser_model):
  """
  Method to parse a JSON audit log and create one or multiple audit events
  from it.
  """
  expected_fields = ('serviceName', 'username', 'impersonator',
      'ipAddress', 'operation', 'eventTime', 'allowed')

  audit_event = dict()
  audit_event['serviceType'] = service_type
  audit_event['extraValues'] = dict()

  for key, value in event_from_audit_log.iteritems():
    key = _get_coerced_key(key, log_parser_model)
    if key == 'eventTime':
      audit_event[key] = int(value)
    elif key == 'allowed':
      if isinstance(value, str):
        # Convert the string to a boolean
        value = value.lower() == 'true'
      elif isinstance(value, int):
        # Convert the int to a boolean
        value = value == 1
      audit_event[key] = value
    elif key in expected_fields:
      audit_event[key] = value
    else:
      audit_event['extraValues'][key] = _get_str_value(value)
    # Sometimes the serviceName in the logs is not what we need (this is the
    # case for Sentry and Solr), so we overwrite it.
    audit_event['serviceName'] = service_name

  return [audit_event]

def _get_str_value(value):
  # Coerce None to empty string
  if value is None:
    value = ''
  elif not isinstance(value, (str, unicode)):
    value = str(value)
  return value

def _get_coerced_key(key, log_parser_model):
  if key in log_parser_model:
    model = log_parser_model.get(key)
    new_field_name = model.get('fieldName')
    if new_field_name:
      key = new_field_name
  return key
