# Copyright (c) 2014 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import logging
import os
import subprocess
import time

from cmf.throttling_logger import ThrottlingLogger

LOG = logging.getLogger(__name__)
THROTTLED_LOG = ThrottlingLogger(LOG)

# Minimum seconds between java location refreshes
JAVA_LOCATION_REFRESH_SECS = 5 * 60

class JavaLocator(object):
  """
  An object to help locate java. This invokes the java_location.sh shell script
  and caches the result for some time.
  """

  def __init__(self, package_dir):
    self._package_dir = package_dir
    self._java_location_valid_until = 0
    self._java_location = None

  def get_java_location(self, now=None):
    """
    Returns the output of java_location.sh. This will return None if the java
    location could not be determined.
    """
    if now is None:
      now = time.time()
    if self._java_location_valid_until < now:
      java_location = self._get_java_location()
      if java_location is not None and java_location != self._java_location:
        THROTTLED_LOG.info("Using java location: '%s'." % java_location)
        self._java_location = java_location
      self._java_location_valid_until = now + JAVA_LOCATION_REFRESH_SECS
    return self._java_location

  def _get_java_location(self):
    try:
      script = os.path.join(self._package_dir, "mgmt/java_location.sh")
      p = subprocess.Popen([script],
                           close_fds=True,
                           stdin=None,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE)
      stdout, stderr = p.communicate()
      if stdout:
        # We want the last line of the output.
        return stdout.splitlines()[-1]
      else:
        THROTTLED_LOG.info("Failed to get Java location: %s" %
                           (repr([stdout, stderr]), ))
    except Exception, e:
      THROTTLED_LOG.exception("Failed to get Java location")
    return None
