# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import logging

from cmf.monitor.abstract_monitor import AbstractMonitor
from cmf.monitor.monitor_properties import ConfParser

LOG = logging.getLogger(__name__)

_HOST = 'host'
_USE_SSL = 'use_ssl'

class AbstractMapReduceMonitor(AbstractMonitor):
  """
  A common AbstractMonitor subclass for MapReduce role monitors.
  """
  def __init__(self, daemon, registry, cgroup_root, cgroup_name, section,\
               metric_port_name, role_type):
    AbstractMonitor.__init__(self, daemon, registry, cgroup_root, cgroup_name, role_type)
    self._section = section
    self._metric_port_name = metric_port_name
    self._metrics_url = None

  # protected
  def _get_metrics_url(self):
    parser = self.conf_parser
    return self._compute_metrics_url(
      parser.get_with_default(self._section, _HOST, None),
      parser.getint_with_default(self._section, self._metric_port_name, None),
      parser.getboolean_with_default(self._section, _USE_SSL, False))

  # protected
  def _compute_metrics_url(self, host, port, use_ssl):
    if host is None or port is None:
      return None
    if use_ssl:
      return "https://%s:%s/metrics?format=json" % (host, port)
    else:
      return "http://%s:%s/metrics?format=json" % (host, port)

  def is_ready_to_report(self):
    if self._pid is None:
      return False
    if not self.is_metrics_url_responding(self._get_metrics_url(), 0.5):
      return False
    return True

  @property
  def metrics_url(self):
    return self._metrics_url

