# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import logging

from cmf.monitor.abstract_monitor import AbstractMonitor
from cmf.monitor.monitor_properties import ConfParser
from cmf.monitor import constants

_HOST = 'host'
_SERVICE_VERSION = 'service_version'
# Added in CM 5.4
_USE_SSL = 'use_ssl'

LOG = logging.getLogger(__name__)

class AbstractSolrMonitor(AbstractMonitor):
  """
  A common AbstractMonitor subclass for Solr role monitors.
  """
  def __init__(self, daemon, registry, cgroup_root, cgroup_name, section, metric_http_port_name,
               metric_https_port_name, role_type):
    AbstractMonitor.__init__(self, daemon, registry, cgroup_root, cgroup_name, role_type)
    self._section = section
    self._metric_http_port_name = metric_http_port_name
    self._metric_https_port_name = metric_https_port_name
    self._metrics_url = None
    self._service_version = None

  # protected
  def _get_metrics_url(self):
    parser = self.conf_parser
    return self._compute_metrics_url(
      parser.get_with_default(self._section, _HOST, None),
      parser.getint_with_default(self._section, self._metric_http_port_name, None),
      parser.getint_with_default(self._section, self._metric_https_port_name, None),
      parser.getboolean_with_default(self._section, _USE_SSL, False))

  # protected
  def _compute_metrics_url(self, host, http_port, https_port, use_ssl):
    if self._service_version is None:
      self._reload_config(self.conf_parser)

    if self._service_version == constants.SERVICE_VERSION_CDH5:
      urlPattern = "%s://%s:%s/solr/jmx"
    else:
      urlPattern = "%s://%s:%s/solr/admin/metrics"

    port = https_port if use_ssl else http_port
    protocol = "https" if use_ssl else "http"

    if host is None or port is None:
      return None
    return urlPattern % (protocol, host, port,)

  def is_ready_to_report(self):
    if self._pid is None:
      return False
    if not self.is_metrics_url_responding(self._get_metrics_url(), 0.5):
      return False
    return True

  @property
  def metrics_url(self):
    return self._metrics_url
