# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import threading

class CountDownLatch(object):

  def __init__(self, count):
    self._count = count
    self._cv = threading.Condition()

  def countdown(self):
    self._cv.acquire()
    if self._count == 0:
      raise Exception('Invalid use of countdown latch. Too many countdowns.')
    self._count = self._count - 1
    if self._count == 0:
      self._cv.notifyAll()          
    self._cv.release()

  def await(self, timeout=None):
    """
    Wait for the count to become zero, using the given timeout.
    Returns True iff the count is zero.
    """
    self._cv.acquire()
    if self._count != 0:
      self._cv.wait(timeout)
    self._cv.release()
    return self._count == 0
