# Copyright (c) 2012 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time
from cmf.monitor.firehoses import Firehoses

class DiskUpdate(object):
  """
  This models the DiskUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self, device, partition, timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp
    self._device = device
    self._partition = partition
    self._metrics = [ ]

  def add_metric(self, metricid, value):
    if value is not None:
      self._metrics.append((metricid, value))
    return self

  def contains_metrics(self):
    return len(self._metrics) > 0

  def get_device(self):
    return self._device

  def is_partition(self):
    return self._partition

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    sio.write(">>  %s" % (self._device))
    sio.write(">>  %s" % (self._partition))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(device=self._device,
                partition=self._partition,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ])
