# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import time
from cmf.monitor.firehoses import Firehoses

class EntityTypeUpdateEntry(object):
  """
  This models the EntityTypeUpdate Avro record and gives it a bit more
  structure.
  """
  def __init__(self, attributes):
    self._attributes = attributes
    self._metrics = []

  def add_metric(self, metricid, value):
    # A value of None causes avro to blow up when we encode the message, so to
    # be safe, we protect against that here.
    if value is not None:
      self._metrics.append((metricid, value))
    return self

  def add_metrics(self, metrics):
    for metricid, value in metrics.iteritems():
      self.add_metric(metricid, value)
    return self

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(attributes=self._attributes,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v)
                          in self._metrics ])


class EntityTypeUpdate(object):
  """
  This models the EntityTypeUpdate Avro record and gives it a bit more
  structure.
  """
  def __init__(self, entityType, deleteOldEntities, createNewEntities):
    self._entityType = entityType
    self._deleteOldEntities = deleteOldEntities
    self._createNewEntities = createNewEntities
    self._entities = []

  def add_entity(self, entity):
    self._entities.append(entity)
    return self

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(entityType=self._entityType,
                entities=[ e.to_avro_dict() for e in self._entities ],
                deleteOldEntities=self._deleteOldEntities,
                createNewEntities=self._createNewEntities)
