# Copyright (c) 2012 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.monitor.firehoses import Firehoses

class FlumeUpdate(object):
  """
  This models the FlumeUpdate Avro record.
  """
  def __init__(self, agent_name, component_type, component_name):
    self._metrics = []
    self._agent_name = agent_name
    self._component_type = component_type
    self._component_name = component_name

  def add_metric(self, metricid, value):
    if metricid is None:
      raise ValueError()
    if value is None:
      raise ValueError()
    self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write("FlumeUpdate(%s, %s, %s):" % (self._agent_name,
                                            self._component_type,
                                            self._component_name))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(componentType=self._component_type,
                componentName=self._component_name,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ],
                agentName=self._agent_name)
