# Copyright (c) 2012 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

from cmf.monitor.json_metrics_utils import ParseError
from cmf.throttling_logger import ThrottlingLogger
from cmf.monitor import schema

# Exhaustive list of all currently exposed Flume metrics, and their
# corresponding entry in our own schema.
FLUME_JSON_METRICS = {
  "source": {
    "StartTime" : schema.SOURCE_START_TIME,
    "StopTime" : schema.SOURCE_STOP_TIME,
    "EventReceivedCount" : schema.EVENT_RECEIVED_COUNT,
    "EventAcceptedCount" : schema.EVENT_ACCEPTED_COUNT,
    "AppendReceivedCount" : schema.APPEND_RECEIVED_COUNT,
    "AppendAcceptedCount" : schema.APPEND_ACCEPTED_COUNT,
    "AppendBatchReceivedCount" : schema.APPEND_BATCH_RECEIVED_COUNT,
    "AppendBatchAcceptedCount" : schema.APPEND_BATCH_ACCEPTED_COUNT,
    "OpenConnectionCount": schema.OPEN_CONNECTIONS
  },
  "channel": {
    "StartTime" : schema.CHANNEL_START_TIME,
    "StopTime" : schema.CHANNEL_STOP_TIME,
    "ChannelSize" : schema.CHANNEL_SIZE,
    "ChannelCapacity" : schema.CHANNEL_CAPACITY,
    "ChannelFillPercentage": schema.CHANNEL_FILL_PERCENTAGE,
    "EventPutAttemptCount" : schema.EVENT_PUT_ATTEMPT_COUNT,
    "EventTakeAttemptCount" : schema.EVENT_TAKE_ATTEMPT_COUNT,
    "EventPutSuccessCount" : schema.EVENT_PUT_SUCCESS_COUNT,
    "EventTakeSuccessCount" : schema.EVENT_TAKE_SUCCESS_COUNT
  },
  "sink": {
    "StartTime" : schema.SINK_START_TIME,
    "StopTime" : schema.SINK_STOP_TIME,
    "ConnectionCreatedCount" : schema.CONNECTION_CREATED_COUNT,
    "ConnectionClosedCount" : schema.CONNECTION_CLOSED_COUNT,
    "ConnectionFailedCount" : schema.CONNECTION_FAILED_COUNT,
    "BatchEmptyCount" : schema.BATCH_EMPTY_COUNT,
    "BatchUnderflowCount" : schema.BATCH_UNDERFLOW_COUNT,
    "BatchCompleteCount" : schema.BATCH_COMPLETE_COUNT,
    "EventDrainAttemptCount" : schema.EVENT_DRAIN_ATTEMPT_COUNT,
    "EventDrainSuccessCount" : schema.EVENT_DRAIN_SUCCESS_COUNT
  }
}

class JsonParser(object):
  def __init__(self, json_dict):
    self._json = json_dict
    self.updates = {}

    for prefixed_name, component_metrics in json_dict.items():
      if not "Type" in component_metrics:
        continue
      component_type = component_metrics["Type"].lower()
      prefix = component_type + "."
      if not prefixed_name.lower().startswith(prefix):
        continue
      if len(prefixed_name) <= len(prefix):
        continue
      component_name = prefixed_name[len(prefix):]
      component_schema = FLUME_JSON_METRICS.get(component_type, None)
      if not component_schema:
        continue
      metricdict = {}
      for metric_json_key, metric_id in component_schema.items():
        if metric_json_key in component_metrics:
          metricdict[metric_id] = float(component_metrics.get(metric_json_key))
      self.updates[(component_type, component_name)] = metricdict

#
# This is what the Flume metrics JSON looks like (pretty-printed for
# readability):
#
SAMPLE = r"""
{
   "SINK.sink1":{
      "BatchCompleteCount":"0",
      "ConnectionFailedCount":"2",
      "EventDrainAttemptCount":"0",
      "ConnectionCreatedCount":"1",
      "Type":"SINK",
      "BatchEmptyCount":"79",
      "ConnectionClosedCount":"0",
      "EventDrainSuccessCount":"0",
      "StopTime":"0",
      "StartTime":"1386636114908",
      "BatchUnderflowCount":"0"
   },
   "SOURCE.source1":{
      "OpenConnectionCount":"0",
      "Type":"SOURCE",
      "AppendBatchAcceptedCount":"0",
      "AppendBatchReceivedCount":"0",
      "EventAcceptedCount":"0",
      "StopTime":"0",
      "AppendReceivedCount":"0",
      "StartTime":"1386636114921",
      "EventReceivedCount":"0",
      "AppendAcceptedCount":"0"
   },
   "CHANNEL.channel1":{
      "EventPutSuccessCount":"0",
      "ChannelFillPercentage":"0.0",
      "Type":"CHANNEL",
      "StopTime":"0",
      "EventPutAttemptCount":"0",
      "ChannelSize":"0",
      "StartTime":"1386636114906",
      "EventTakeSuccessCount":"0",
      "ChannelCapacity":"10000",
      "EventTakeAttemptCount":"80"
   }
}
"""
