# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time
from cmf.monitor.firehoses import Firehoses

class FsUpdate(object):
  """
  This models the FsUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self,
               mountpoint,
               mountOptions,
               partition,
               fstype,
               timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp
    self._mountpoint = mountpoint
    self._mountOptions = mountOptions
    self._partition = partition
    self._fstype = fstype
    self._metrics = [ ]

  def add_metric(self, metricid, value):
    self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    sio.write(">>  %s [%s] (%s) fstype: %s" % (self._mountpoint,
                                               self._mountOptions,
                                               self._partition,
                                               self._fstype))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(mountpoint=self._mountpoint,
                mountOptions=self._mountOptions,
                partition=self._partition,
                fstype=self._fstype,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ])
