# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import logging
import os

from cmf.monitor import constants
from cmf.monitor.generic.adapter import Adapter
from cmf.monitor.jmx_beans import JsonParserJmx
from cmf.monitor.json_metrics_utils import add_derived_jvm_metrics_jmx
from cmf.monitor.json_metrics_utils import add_metrics_jmx
from cmf.monitor.json_metrics_utils import add_pause_metrics

LOG = logging.getLogger('HadoopAdapter')

class HadoopAdapter(Adapter):
  """
  Base class for all hadoop-based service role adapters. This exists to provide
  utility methods and to ensure consistency in the handling of common metrics.
  """

  # Added in CM 5.1
  _USE_SSL = 'use_ssl'

  def __init__(self,
               service_type,
               role_type,
               safety_valve,
               http_port_key=None,
               https_port_key=None):
    Adapter.__init__(self, service_type, role_type, safety_valve)
    self._http_port_key = http_port_key
    self._https_port_key = https_port_key

  def get_metrics_url(self, conf):
    if conf is None:
      raise Exception("A configuration is required!")
    if self._http_port_key is None:
      return None
    try:
      host = conf.get(self.section, Adapter._HOST)
      if host is None:
        LOG.error("%s entry missing from %s" % (Adapter._HOST,
                                                os.path.abspath(conf.path)))
        return None
      if conf.getboolean_with_default(
          self.section, HadoopAdapter._USE_SSL, False):
        url_format = "https://%s:%s/jmx"
        port_key = self._https_port_key
      else:
        url_format = "http://%s:%s/jmx"
        port_key = self._http_port_key
      port = conf.getint(self.section, port_key)
      if port is None:
        LOG.error("%s entry missing from %s" % (port_key,
                                                os.path.abspath(conf.path)))
        return None
      return  url_format % (host, port)
    except:
      LOG.exception("Error parsing %s" % os.path.abspath(conf.path))
      return None

  def parse_metrics_from_url(self, conf, json):
    return JsonParserJmx(json, self.role_type)

  def add_webserver_metrics(self, version, update, metrics, accessors):
    add_derived_jvm_metrics_jmx(update, metrics)
    if version == constants.SERVICE_VERSION_CDH5:
      version_string = "cdh5"
    elif version == constants.SERVICE_VERSION_CDH6:
      version_string = "cdh6"
    else:
      raise Exception("Unknown %s version: %s" % (self._role_type, version))
    add_metrics_jmx(update, metrics, accessors, self._role_type, version_string)
    add_pause_metrics(update, metrics)
