# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import os

from cmf.monitor import constants
from cmf.monitor.generic.hadoop_adapter import HadoopAdapter

class ResourceManagerAdapter(HadoopAdapter):
  _RESOURCEMANAGER_HTTP_PORT = 'resourcemanager_webserver_port'
  # Added in CM 5.1
  _RESOURCEMANAGER_HTTPS_PORT = 'resourcemanager_webserver_https_port'

  def __init__(self, safety_valve):
    HadoopAdapter.__init__(
        self,
        "YARN",
        "RESOURCEMANAGER",
        safety_valve,
        http_port_key=ResourceManagerAdapter._RESOURCEMANAGER_HTTP_PORT,
        https_port_key=ResourceManagerAdapter._RESOURCEMANAGER_HTTPS_PORT)

  def get_metrics_sample(self, version):
    if version == constants.SERVICE_VERSION_CDH5:
      return open(os.path.join(os.path.dirname(__file__), "../samples/yarn_rm_cdh5"), 'r').read()
    else:
      return None

class JobHistoryAdapter(HadoopAdapter):
  _JOBHISTORY_HTTP_PORT = 'mapreduce_jobhistory_webapp_address'
  # Added in CM 5.1
  _JOBHISTORY_HTTPS_PORT = 'mapreduce_jobhistory_webapp_https_address'

  def __init__(self, safety_valve):
    HadoopAdapter.__init__(
        self,
        "YARN",
        "JOBHISTORY",
        safety_valve,
        http_port_key=JobHistoryAdapter._JOBHISTORY_HTTP_PORT,
        https_port_key=JobHistoryAdapter._JOBHISTORY_HTTPS_PORT)

  def get_metrics_sample(self, version):
    if version == constants.SERVICE_VERSION_CDH5:
      return open(os.path.join(os.path.dirname(__file__), "../samples/yarn_jhs_cdh5"), 'r').read()
    else:
      return None

class NodeManagerAdapter(HadoopAdapter):
  _NODEMANAGER_HTTP_PORT = 'nodemanager_webserver_port'
  # Added in CM 5.1
  _NODEMANAGER_HTTPS_PORT = 'nodemanager_webserver_https_port'

  def __init__(self, safety_valve):
    HadoopAdapter.__init__(
        self,
        "YARN",
        "NODEMANAGER",
        safety_valve,
        http_port_key=NodeManagerAdapter._NODEMANAGER_HTTP_PORT,
        https_port_key=NodeManagerAdapter._NODEMANAGER_HTTPS_PORT)
    self._descendants_proc_metrics_flag = True

  def get_metrics_sample(self, version):
    if version == constants.SERVICE_VERSION_CDH5:
      return open(os.path.join(os.path.dirname(__file__), "../samples/yarn_nm_cdh5"), 'r').read()
    else:
      return None
