# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

import logging
import os
import datetime

from cmf.monitor import schema
from cmf.throttling_logger import ThrottlingLogger
from M2Crypto import X509

LOG = logging.getLogger(__name__)
THROTTLED_LOG = ThrottlingLogger(LOG)


class CertificateMetrics(object):
  def __init__(self, agentConfig):
    self._agentConfig = agentConfig
    self._cert_file = self._agentConfig.client_cert_file
    pass

  def add_certificate_metrics(self, update):
    """Adds certificate expiry metrics to the update"""
    if self._cert_file is None:
        THROTTLED_LOG.debug("No cert monitoring: cert file config is undefined")
        return
    if not os.path.exists(self._cert_file):
        THROTTLED_LOG.error("No cert monitoring: cert file does not exist: " + self._cert_file)
        return
    try:
        daysToExpiry = self._get_cert_days_left(self._cert_file)
    except Exception as e:
        THROTTLED_LOG.error("Could not calculate expiry date for cert file: " + self._cert_file)
        THROTTLED_LOG.error(e)
        return
    THROTTLED_LOG.debug("Days till agent cert expiry: " + str(daysToExpiry))
    update.add_metric(schema.AGENT_CERT_EXPIRY, int(daysToExpiry))

  def _get_cert_days_left(self, cert_path):
    cert = X509.load_cert(cert_path)
    return (cert.get_not_after().get_datetime().date() - datetime.datetime.now().date()).days

