# Copyright (c) 2017 Cloudera, Inc. All rights reserved.

import logging

from cmf.monitor import schema
from cmf.throttling_logger import ThrottlingLogger

LOG = logging.getLogger(__name__)
THROTTLED_LOG = ThrottlingLogger(LOG)

_PROC_ENTROPY = "/proc/sys/kernel/random/entropy_avail"

class EntropyMonitor(object):
  def __init__(self,
               proc_entropy = _PROC_ENTROPY):
    self._entropy_file = None
    try:
      entropy= open(proc_entropy)
      self._entropy_file = proc_entropy
      entropy.close()
    except IOError, ioex:
      THROTTLED_LOG.warning("File '%s' couldn't be opened for entropy count "
                  "collection, error=%s" % (proc_entropy, ioex.errno))
    except Exception, e:
      THROTTLED_LOG.exception("Unknown error opening file '%s', error: %s" % (proc_entropy, e))

  def add_entropy_metrics(self, update):
    """Adds entropy count metric to the update"""
    if self._entropy_file is None:
      return

    try:
      try:
        entropy = open(self._entropy_file)
      except IOError, ioex:
        THROTTLED_LOG.warn("File '%s' couldn't be opened, error=%s"
                         % (self._entropy_file, ioex.errno))
        return

      try:
        entropy_available = entropy.readline().strip()
      finally:
        entropy.close()

      if entropy_available is None or len(entropy_available) == 0:
        THROTTLED_LOG.debug("entropy information is not available on this host")
        return

      update.add_metric(schema.AVAILABLE_ENTROPY, int(entropy_available))
    except Exception, e:
      THROTTLED_LOG.exception("Failed to collect entropy metrics: %s", e)
