# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

from cmf.monitor import schema
from cmf.monitor.fs_update import FsUpdate

class Filesystem(object):
  """Represents information related to a filesystem"""
  def __init__(self, fs_map, mountpoint, partition, options, nodev, fstype):
    self._fs_map = fs_map
    self._mountpoint = mountpoint
    self._partition = partition
    self._options = options
    self._nodev = nodev
    self._fstype = fstype

    self._usage_info = None

  @property
  def mountpoint(self):
    return self._mountpoint

  @property
  def partition(self):
    return self._partition

  @property
  def options(self):
    return self._options

  @property
  def nodev(self):
    return self._nodev

  @property
  def fstype(self):
    return self._fstype

  def collect(self):
    self._usage_info = self._fs_map.disk_usage(self._mountpoint)

  def get_fs_update(self):
    update = FsUpdate(self._mountpoint, self._options, self._partition, self._fstype)
    if self._usage_info is not None:
      update.add_metric(schema.CAPACITY, self._usage_info['total'])
      update.add_metric(schema.CAPACITY_USED, self._usage_info['used'])
      update.add_metric(schema.CAPACITY_FREE, self._usage_info['free'])
    return update
