# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import time

class ImpalaRuntimeProfile(object):
  """
  This models the AgentAvroImpalaRuntimeProfile record and gives it a bit more
  structure.
  """
  def __init__(self, compressed_runtime_profile = None):
      self._compressed_runtime_profile = compressed_runtime_profile

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(compressedRuntimeProfile=self._compressed_runtime_profile)

class ImpalaQueryUpdate(object):
  """
  This models the ImpalaQueryUpdate record and gives it a bit more structure.
  """
  def __init__(self,
               timestamp=None,
               role_name=None,
               service_name=None,
               impala_version=None,
               executing_query_profiles=[],
               completed_query_profiles=[]):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp
    self._role_name = role_name
    self._service_name = service_name
    self._impala_version = impala_version
    self._executing_query_profiles = executing_query_profiles
    self._completed_query_profiles = completed_query_profiles

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(ts_secs=long(self._timestamp),
                roleName=self._role_name,
                serviceName=self._service_name,
                impalaVersion=self._impala_version,
                executingQueryProfiles=[ p.to_avro_dict() for p in 
                                         self._executing_query_profiles ],
                completedQueryProfiles=[ p.to_avro_dict() for p in
                                         self._completed_query_profiles ])

