# Copyright (c) 2012 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time
from cmf.monitor import schema
from cmf.monitor.constants import DUPLEX_UNKNOWN
from cmf.monitor.firehoses import Firehoses

class NicUpdate(object):
  """
  This models the NicUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self,
               iface,
               speed=None,
               duplex_mode=None,
               bytes_receive=None,
               packets_receive=None,
               errs_receive=None,
               drop_receive=None,
               frame_receive=None,
               bytes_transmit=None,
               packets_transmit=None,
               errs_transmit=None,
               drop_transmit=None,
               colls_transmit=None,
               timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp
    self._iface = iface
    self._duplex_mode = duplex_mode
    if self._duplex_mode is None:
      self._duplex_mode = DUPLEX_UNKNOWN
    self._metrics = [ ]
    self.add_metric(schema.SPEED, speed)
    self.add_metric(schema.BYTES_RECEIVE, bytes_receive)
    self.add_metric(schema.PACKETS_RECEIVE, packets_receive)
    self.add_metric(schema.ERRS_RECEIVE, errs_receive)
    self.add_metric(schema.DROP_RECEIVE, drop_receive)
    self.add_metric(schema.FRAME_RECEIVE, frame_receive)
    self.add_metric(schema.BYTES_TRANSMIT, bytes_transmit)
    self.add_metric(schema.PACKETS_TRANSMIT, packets_transmit)
    self.add_metric(schema.ERRS_TRANSMIT, errs_transmit)
    self.add_metric(schema.DROP_TRANSMIT, drop_transmit)
    self.add_metric(schema.COLLS_TRANSMIT, colls_transmit)

  def add_metric(self, metricid, value):
    if value is not None:
      self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    sio.write(">>  %s" % (self._iface))
    sio.write(">>  %s" % (self._duplex_mode))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(iface=self._iface,
                duplexMode=self._duplex_mode,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ])
