# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import logging
import threading

LOG = logging.getLogger(__name__)

class MonitorRegistry(object):
  """
  A registry for monitor objects to register that they are active and would
  like to participate in the periodic firehose report.
  """
  def __init__(self):
    self._registry = set()
    self._registry_lock = threading.Lock()

  def register(self, monitor):
    """Register the given monitor to be included in the periodic report."""
    self._registry_lock.acquire()
    try:
      self._registry.add(monitor)
      LOG.debug("Registered monitor '%s'" % (monitor,))
    finally:
      self._registry_lock.release()

  def unregister(self, monitor):
    """Register the given monitor to be included in the periodic report."""
    self._registry_lock.acquire()
    try:
      if monitor in self._registry:
        self._registry.remove(monitor)
        LOG.debug("Unregistered monitor '%s'" % (monitor,))
    finally:
      self._registry_lock.release()

  def get_registry(self):
    """Return a copy of the registry"""
    self._registry_lock.acquire()
    try:
      return self._registry.copy()
    finally:
      self._registry_lock.release()
