# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time
from cmf.monitor.firehoses import Firehoses

class RoleDirUpdate(object):
  """
  This models the RoleDirUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self,
               path,
               mountpoint,
               mountOptions,
               partition,
               directoryType,
               permissions,
               owner,
               group,
               fstype,
               timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp

    self._path = path
    self._mountpoint = mountpoint
    self._mountOptions = mountOptions
    self._partition = partition
    self._directoryType = directoryType
    self._permissions = permissions
    self._owner = owner
    self._group = group
    self._fstype = fstype
    self._metrics = [ ]

  def add_metric(self, metricid, value):
    self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    sio.write(">>  %s (on %s %s [%s]) type: %s permissions: %s owner: %s group: %s"
              "fstype: %s" %
              (self._path, self._mountpoint, self._partition, self._mountOptions,
               self._directoryType, self._permissions, self._owner, self._group,
               self._fstype))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(path=self._path,
                mountpoint=self._mountpoint,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ],
                mountOptions=self._mountOptions,
                partition=self._partition,
                directoryType=self._directoryType,
                permissions=self._permissions,
                owner=self._owner,
                group=self._group,
                fstype=self._fstype)
