# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time
from cmf.monitor.firehoses import Firehoses

class ServiceUpdate(object):
  """
  This models the ServiceUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self, name, service_type, timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp
    self._name = name
    self._service_type = service_type
    self._metrics = [ ]

  def add_metric(self, metricid, value):
    # A value of None causes avro to blow up when we encode the message, so to
    # be safe, we protect against that here.
    if value is not None:
      self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(ts_secs=long(self._timestamp),
                service_name=self._name,
                service_type=self._service_type,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ])

