# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.monitor.firehoses import Firehoses

class SolrUpdate(object):
  """
  This models the SolrUpdate Avro record.
  """
  def __init__(self, replica_name):
    self._metrics = []
    if isinstance(replica_name, list):
      self._set_components(replica_name)
      return
    (self._collection_name, self._shard_name, rep) = replica_name.rsplit('_', 2)
    self._replica_name = replica_name

  def _set_components(self, replica_components):
    (self._collection_name, self._shard_name, rep) = replica_components
    self._replica_name = "_".join(replica_components)

  def add_metric(self, metricid, value):
    if metricid is None:
      raise ValueError()
    if value is None:
      raise ValueError()
    self._metrics.append((metricid, value))
    return self

  def get_replica_name(self):
    return self._replica_name

  def __str__(self):
    sio = StringIO()
    sio.write("SolrUpdate(%s_%s_%s):" % (self._collection_name, \
                                         self._shard_name, \
                                         self._replica_name))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(collectionName=self._collection_name,
                shardName=self._shard_name,
                replicaName=self._replica_name,
                metrics=[ dict(id=k, value=Firehoses.normalize_metric_value(v))
                          for (k, v) in self._metrics ])
