# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import logging
import threading
import time

LOG = logging.getLogger(__name__)

class StoppableThread(threading.Thread):
  """
  A stoppable thread that periodically executes some action.
  """
  def __init__(self, name, interval, fn, *args, **kwargs):
    threading.Thread.__init__(self, name=name)
    self.setDaemon(True)
    self._interval = interval
    self._fn = fn
    self._args = args
    self._kwargs = kwargs
    self._should_stop = False

  def run(self):
    LOG.debug("Starting %s" % (self.getName(),))
    while not self._should_stop:
      start = time.time()
      try:
        self._fn(*self._args, **self._kwargs)
      except Exception, _:
        # Only catch Exception, and allow InterruptException through
        LOG.exception("Exception in thread '%s'" % (self.getName(),))

      sleep = self._interval - (time.time() - start)
      if sleep >= 0:
        time.sleep(sleep)

    LOG.debug("Stopping %s" % (self.getName(),))

  def should_stop(self):
    """Instruct the thread to stop at the next loop"""
    self._should_stop = True

  def set_interval(self, sec):
    self._interval = sec

  def get_interval(self):
    return self._interval

  def set_name(self, val):
    self.setName(val)
