# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

import urllib2
import logging

LOG = logging.getLogger(__name__)

# URL used to access AWS EC2 instance metadata
AWS_EC2_METADATA_ENDPOINT = "169.254.169.254/2012-01-12/meta-data/"
# Ending of public hostnames on EC2 used to determine we run on EC2
AWS_EC2_PUBLIC_HOSTNAME_SUFFIX = ".amazonaws.com"
TIMEOUT_SECONDS = 2

# CloudStatus fields
PUBLIC_HOST_NAME = "public_host_name"
PUBLIC_IP_ADDRESS = "public_ip_address"
CLOUD_PROVIDER = "cloud_provider"

def _create_cloud_status_dict():
  # The cloud info dict must match CloudStatus in the heartbeat
  return {
      PUBLIC_HOST_NAME : None,
      PUBLIC_IP_ADDRESS : None,
      CLOUD_PROVIDER : None
      }

def _get_aws_ec2_metadata(uri):
  url = 'http://%s/%s/' % (AWS_EC2_METADATA_ENDPOINT, uri)
  try:
    return urllib2.urlopen(url,timeout=TIMEOUT_SECONDS).read()
  except:
    return None

def _get_aws_ec2_info():
  public_hostname = _get_aws_ec2_metadata("public-hostname")
  if public_hostname is None or not public_hostname.endswith(AWS_EC2_PUBLIC_HOSTNAME_SUFFIX):
    return None
  cloud_info = _create_cloud_status_dict()
  cloud_info[PUBLIC_HOST_NAME] = public_hostname
  cloud_info[PUBLIC_IP_ADDRESS] = _get_aws_ec2_metadata("public-ipv4")
  cloud_info[CLOUD_PROVIDER] = "aws-ec2"
  return cloud_info

def get_public_cloud_info():
  """Determines if the agent is running on a known public cloud and returns
     the public ip and hostname for the instance.
     Currently only supports Amazon EC2.
  """
  return _get_aws_ec2_info()

