"""
Copyright (c) 2011-2016 Cloudera, Inc. All rights reserved.

Contains classes for managing parsed config files.
"""
from __future__ import absolute_import

# pylint: disable=import-error,broad-except,invalid-name
import click
import requests
import telnetlib
import socket
import os.path

@click.group()
@click.pass_context
@click.option("-H", "--host", help="Host to connect to.", default="localhost")
@click.option("-p", "--port", help="Port to connect to (default=9001).", default=9001)
def main(ctx, host, port):
  """
  Remote console access to CM Agent.

  The agent provides a remote console environment within which one may execute an interactive
  session or upload a batch code fragment to.

  The environment persists between sessions/batch jobs, thus setting a variable in one session
  will be remembered and available in the next.

  The environment provides the following symbols:

  LOG : A logger instance.

  output(fmt, *pargs) : A function that prints a string to be returned as output (batch only).

  agent: The Agent object instance.
  """
  ctx.obj = {}
  ctx.obj['host'] = host
  ctx.obj['port'] = port

@main.command()
@click.option("-s", "--source", help="Data file to transmit", default="-", type=click.File("rb"))
def upload(ctx, source):
  """
  Upload and execute a batch script into the remote console environment.

  Calls to "output()" from the script will be reflected in the output of this command.
  """
  host = ctx.obj['host']
  port = ctx.obj['port']
  data = source.read(2<<20)
  out = requests.put("http://%s:%s/console/upload" % (host, port),
                     data=data)
  print out.content


class UNIXTelnet(telnetlib.Telnet):
  """
  Run Telnet protocol over a UNIX socket.
  """
  # pylint: disable=protected-access
  def open(self, host, port=0, timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
    self.eof = 0
    self.host = host
    self.port = port
    self.timeout = timeout
    self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    self.sock.connect(host)

@main.command()
@click.pass_context
def interactive(ctx):
  """
  Runs an interactive session of the python interpreter in the agent process.

  (This will use the "rlwrap" tool if available for enabling dynamic command line editing.)
  """
  host = ctx.obj['host']
  port = ctx.obj['port']


  cookies = {'token': open(os.path.join(ctx.obj['cfg'].paths.auth_token)).read()}
  out = requests.put("http://%s:%s/python_console" % (host, port), cookies=cookies)
  addr = out.content.strip()
  t = UNIXTelnet(addr)
  t.mt_interact()

if __name__ == "__main__":
  # pylint: disable=no-value-for-parameter
  main()
