# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from cmf.audit.navigator_logs import NoneNavigatorLogFile, NonexistentNavigatorLogFile
from cmf.audit.util import make_audit_plugin
from cmf_test.audit.test_audit_plugin import TestAuditPlugin, ROLE_CONFIG_DIR, AUDIT_LOG_DIR
from nose.tools import assert_equal, assert_false, assert_true

_REGIONSERVER_ROLE_CONFIG_DIR = "/tmp/hbase-regionserver"
_AUDIT_LOG_DIR = "/tmp/audit_log"

class TestRegionServerAuditPlugin(TestAuditPlugin):
  """
  This class tests methods in the Hbase RegionServerAuditPlugin class.
  """

  def __init__(self):
    super(TestRegionServerAuditPlugin, self).__init__("HBASE-1", "HBASE",
      "HBASE-1-REGIONSERVER-bd6408bdfcb0aca08a18af8853057f19", "REGIONSERVER")

  def test_audit_plugin_refresh_hbase_regionserver(self):
    """
    Test if hbase regionserver audit plugin's refresh method picks up all the
    properties for hbase regionserver role.
    """
    plugin = make_audit_plugin("hbase-REGIONSERVER", {'conf_dir': _REGIONSERVER_ROLE_CONFIG_DIR})

    plugin.refresh(ROLE_CONFIG_DIR)
    plugin.immediate_refresh()
    assert_equal(plugin.nav_conf.log_dir, AUDIT_LOG_DIR)
    assert_equal(plugin.get_role_type(), "REGIONSERVER")
    assert_equal(plugin.get_service_name(), "HBASE-1")
    assert_equal(plugin.get_ipc_request_name(), "writeHBaseAuditEvents")
    from cmf.audit.nop_audit_parser import parse_audit_event
    assert_true(plugin.get_log_entry_parser() is parse_audit_event)

    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 4)

    for pipeline in pipelines:
      name = pipeline.get_name()
      if name.endswith("TP"):
        rname = name.rstrip("TP")
      else:
        rname = name
      assert_equal(pipeline._get_log_prefix(self.get_role_name()),
                   "HBASE-1-REGIONSERVER-bd6408bdfcb0aca08a18af8853057f19-" + rname)
      assert_equal(pipeline._set_wal_filename(plugin.get_role_type()),
                   "regionserver_" + name.lower() + "_audit_wal")
      assert_false(pipeline.get_nav_logs_dir() is None)
      assert_true(type(pipeline.get_current_nav_log()) is NoneNavigatorLogFile)

      file_name, offset = pipeline.get_wal().get_file_info()
      assert_true(file_name is None)
      assert_true(offset is None)
      pipeline.switch_log("non-existing-file")
      assert_true(type(pipeline.get_current_nav_log()) is NonexistentNavigatorLogFile)
