# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from cmf.audit.navigator_logs import NoneNavigatorLogFile, NonexistentNavigatorLogFile
from cmf.audit.util import make_audit_plugin
from cmf_test.audit.test_audit_plugin import TestAuditPlugin, ROLE_CONFIG_DIR, AUDIT_LOG_DIR
from nose.tools import assert_equal, assert_false, assert_true

class TestHiveServer2AuditPlugin(TestAuditPlugin):
  """
  This class tests methods in the Hive HiveServer2AuditPlugin class.
  """

  def __init__(self):
    super(TestHiveServer2AuditPlugin, self).__init__("HIVE-1", "HIVE",
      "HIVE-1-HIVESERVER2-8289bf63ac31fd75423fb00cf25e8e3b", "HIVESERVER2")

  def test_audit_plugin_refresh_hive_hs2(self):
    """
    Test if hive HiveServer2 audit plugin's refresh method picks up all the
    properties for hive hs2 role.
    """
    plugin = make_audit_plugin("hive-HIVESERVER2", {'conf_dir': ROLE_CONFIG_DIR})

    plugin.refresh(ROLE_CONFIG_DIR)
    plugin.immediate_refresh()
    assert_equal(plugin.nav_conf.log_dir, AUDIT_LOG_DIR)
    assert_equal(plugin.get_role_type(), "HIVESERVER2")
    assert_equal(plugin.get_service_name(), "HIVE-1")
    assert_equal(plugin.get_ipc_request_name(), "writeHiveAuditEvents")
    from cmf.audit.nop_audit_parser import parse_audit_event
    assert_true(plugin.get_log_entry_parser() is parse_audit_event)

    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)

    for pipeline in pipelines:
      name = pipeline.get_name()
      if name.endswith("TP"):
        rname = name.rstrip("TP")
      else:
        rname = name
      assert_equal(pipeline._get_log_prefix(self.get_role_name()),
                   "HIVE-1-HIVESERVER2-8289bf63ac31fd75423fb00cf25e8e3b-" + rname)
      assert_equal(pipeline._set_wal_filename(plugin.get_role_type()),
                   "hiveserver2_" + name.lower() + "_audit_wal")
      assert_false(pipeline.get_nav_logs_dir() is None)
      assert_true(type(pipeline.get_current_nav_log()) is NoneNavigatorLogFile)

      file_name, offset = pipeline.get_wal().get_file_info()
      assert_true(file_name is None)
      assert_true(offset is None)
      pipeline.switch_log("non-existing-file")
      assert_true(type(pipeline.get_current_nav_log()) is NonexistentNavigatorLogFile)
