# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from cmf.audit.navigator_logs import NoneNavigatorLogFile, NonexistentNavigatorLogFile
from cmf.audit.util import make_audit_plugin
from cmf_test.audit.test_audit_plugin import TestAuditPlugin, ROLE_CONFIG_DIR, AUDIT_LOG_DIR
from nose.tools import assert_equal, assert_false, assert_true

class TestBeeswaxServerAuditPlugin(TestAuditPlugin):
  """
  This class tests methods in the Hue BeeswaxServerAuditPlugin class.
  """

  def __init__(self):
    super(TestBeeswaxServerAuditPlugin, self).__init__("HUE-2", "HUE",
      "HUE-2-BEESWAX_SERVER-6a3fbae1c4f9314c9adfdc220bbe8f5c", "BEESWAX_SERVER")

  def test_audit_plugin_refresh_hue_beeswaxserver(self):
    """
    Test if hue beeswaxserver audit plugin's refresh method picks up all the
    properties for hive metastore plugin configuration.
    """
    plugin = make_audit_plugin("hue-BEESWAX_SERVER", {'conf_dir': ROLE_CONFIG_DIR})

    plugin.refresh(ROLE_CONFIG_DIR)
    plugin.immediate_refresh()
    assert_equal(plugin.nav_conf.log_dir, AUDIT_LOG_DIR)
    assert_equal(plugin.get_role_type(), "BEESWAX_SERVER")
    assert_equal(plugin.get_service_name(), "HUE-2")
    assert_equal(plugin.get_ipc_request_name(), "writeHiveAuditEvents")
    from cmf.audit.nop_audit_parser import parse_audit_event
    assert_true(plugin.get_log_entry_parser() is parse_audit_event)

    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)

    for pipeline in pipelines:
      name = pipeline.get_name()
      if name.endswith("TP"):
        rname = name.rstrip("TP")
      else:
        rname = name
      assert_equal(pipeline._get_log_prefix(self.get_role_name()),
                   "HUE-2-BEESWAX_SERVER-6a3fbae1c4f9314c9adfdc220bbe8f5c-" + rname)
      assert_equal(pipeline._set_wal_filename(plugin.get_role_type()),
                   "beeswax_server_" + name.lower() + "_audit_wal")
      assert_false(pipeline.get_nav_logs_dir() is None)
      assert_true(type(pipeline.get_current_nav_log()) is NoneNavigatorLogFile)

      file_name, offset = pipeline.get_wal().get_file_info()
      assert_true(file_name is None)
      assert_true(offset is None)
      pipeline.switch_log("non-existing-file")
      assert_true(type(pipeline.get_current_nav_log()) is NonexistentNavigatorLogFile)
