# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
from cmf.audit.navigator_logs import NoneNavigatorLogFile, NonexistentNavigatorLogFile
from cmf.audit.util import make_audit_plugin
from cmf.audit.impala.impalad_audit_parser import parse_impalad_audit_event
from cmf_test.audit.test_audit_plugin import TestAuditPlugin, ROLE_CONFIG_DIR, AUDIT_LOG_DIR
from nose.tools import assert_equal, assert_false, assert_true

_IMPALAD_ROLE_CONFIG_DIR = "/tmp/impalad"
_AUDIT_LOG_DIR = "/tmp/audit_log"

class TestImpaladAuditPlugin(TestAuditPlugin):
  """
  This class tests methods in the Impalad AuditPlugin class.
  """

  def __init__(self):
    super(TestImpaladAuditPlugin, self).__init__("IMPALA-1", "IMPALA",
      "IMPALA-1-IMPALAD-abee5a758942d6c689c7a5e7dd2da179", "IMPALAD")

  def test_audit_plugin_refresh_impalad(self):
    """
    Test if impalad audit plugin's refresh method picks up all the properties
    for impalad role.
    """
    plugin = make_audit_plugin("impala-IMPALAD", {'conf_dir': ROLE_CONFIG_DIR})

    plugin.refresh(ROLE_CONFIG_DIR)
    plugin.immediate_refresh()
    assert_equal(plugin.nav_conf.log_dir, AUDIT_LOG_DIR)
    assert_equal(plugin.get_role_type(), "IMPALAD")
    assert_equal(plugin.get_service_name(), "IMPALA-1")
    assert_equal(plugin.get_ipc_request_name(), "writeImpalaAuditEvents")
    assert_true(plugin.get_log_entry_parser() is parse_impalad_audit_event)


    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)

    for pipeline in pipelines:
      assert_equal(pipeline._get_log_prefix(self.get_role_name()),
                   "impala_audit_event_log_")
      name = pipeline.get_name()
      if name.endswith("TP"):
        assert_equal(pipeline._set_wal_filename(plugin.get_role_type()),
                       "impalad_tp_audit_wal")
      else:
        assert_equal(pipeline._set_wal_filename(plugin.get_role_type()),
                       "impalad_audit_wal")
      assert_false(pipeline.get_nav_logs_dir() is None)
      assert_true(type(pipeline.get_current_nav_log()) is NoneNavigatorLogFile)

      file_name, offset = pipeline.get_wal().get_file_info()
      assert_true(file_name is None)
      assert_true(offset is None)
      pipeline.switch_log("non-existing-file")
      assert_true(type(pipeline.get_current_nav_log()) is NonexistentNavigatorLogFile)
