# Copyright (c) 2014 Cloudera, Inc. All rights reserved.

from cmf.audit.util import make_audit_plugin
from cmf_test.audit.test_audit_plugin import TestAuditPlugin, ROLE_CONFIG_DIR
from nose.tools import assert_equal, assert_false, assert_true

class TestAuditMetrics(TestAuditPlugin):
  """
  Class to test if audit metrics are generated correctly.
  """

  def test_audit_plugin_metrics(self):
    """
    Test if audit plugin can parse metrics correctly.
    """
    plugin = make_audit_plugin("hdfs-NAMENODE")
    plugin.refresh(ROLE_CONFIG_DIR)
    plugin.immediate_refresh()

    # check if metrics can be parsed correctly
    class RoleUpdate(object):
      def __init__(self, metrics):
        self._metrics = metrics

      def add_metric(self, id, value):
        self._metrics[id] = value

    metrics = dict()
    role_update = RoleUpdate(metrics)
    plugin.add_metrics(role_update)

    assert_equal(metrics[25678], 13.0)
    assert_equal(metrics[25679], 478.0)
    assert_equal(metrics[25680], 2.0)
    assert_equal(metrics[25681], 0.0)
    assert_equal(metrics[25682], 11.0)

  def test_agent_audit_metrics(self):
    """
    Test if metrics generated by agent is handled correctly.
    """
    plugin = make_audit_plugin("hdfs-NAMENODE")
    plugin.refresh(ROLE_CONFIG_DIR)
    plugin.immediate_refresh()
    audit_metrics = plugin.get_agent_nav_metrics()
    audit_metrics.increment_send_failures()
    audit_metrics.increment_bytes_sent(1000L)
    audit_metrics.increment_events_sent(15L)

    # check if metrics can be parsed correctly
    class RoleUpdate(object):
      def __init__(self, metrics):
        self._metrics = metrics

      def add_metric(self, id, value):
        self._metrics[id] = value

    metrics = dict()
    role_update = RoleUpdate(metrics)
    audit_metrics.add_metrics(role_update, plugin.get_nav_pipelines())

    assert_equal(metrics[25683], 1L)
    assert_equal(metrics[25684], 15L)
    assert_equal(metrics[25685], 1000L)
