# Copyright (c) 2014 Cloudera, Inc. All rights reserved.

from cmf.audit.generic_audit_parser import parse_generic_audit_event

from nose.tools import assert_equal

ROLE_CONFIG_DIR = "/tmp/audit_role_config_dir"
AUDIT_LOG_DIR = "/tmp/audit_log"

class TestGenericAuditParser(object):
  """
  Class that parses generic events
  """
  def test_generic_audit_parser(self):
    """
    Test if generic audit parser puts all extra fields into the extraValues map.
    """
    service_type = 'type'
    service_name = 'name'
    username = 'abcUser'
    impersonator = 'abcImpersonator'
    ip_address = '1.2.3.4'
    operation = 'abcOperation'
    event_time = 123
    allowed = True

    extra_key_1 = 'key1'
    extra_value_1 = 'abc'

    extra_key_2 = 'key2'
    extra_value_2 = 123

    extra_key_3 = 'key3'
    extra_value_3 = u'\ufffd'

    audit_event = {
      'serviceName': 'otherName',
      'username': username,
      'impersonator': impersonator,
      'ipAddress': ip_address,
      'operation': operation,
      'eventTime': event_time,
      'allowed': allowed,
      extra_key_1: extra_value_1,
      extra_key_2: extra_value_2,
      extra_key_3: extra_value_3
    }

    result = parse_generic_audit_event(service_type, service_name, audit_event, {})

    result = result[0]
    assert_equal(service_type, result['serviceType'])
    assert_equal(service_name, result['serviceName'])
    assert_equal(username, result['username'])
    assert_equal(impersonator, result['impersonator'])
    assert_equal(ip_address, result['ipAddress'])
    assert_equal(operation, result['operation'])
    assert_equal(event_time, result['eventTime'])
    assert_equal(allowed, result['allowed'])

    assert_equal(extra_value_1, result['extraValues'][extra_key_1])
    # ints are converted to strings
    assert_equal(str(extra_value_2), result['extraValues'][extra_key_2])
    assert_equal(extra_value_3, result['extraValues'][extra_key_3])

    audit_event['allowed'] = False
    result = parse_generic_audit_event(service_type, service_name, audit_event, {})
    result = result[0]
    assert_equal(False, result['allowed'])

  def test_generic_audit_parser_model(self):
    """
    Test if generic audit parser modifies fields specified in the
    log_parser_model to the format needed.
    """
    service_type = 'type'
    service_name = 'cmfServiceName'
    username = 'abcUser'
    impersonator = 'abcImpersonator'
    ip_address = '1.2.3.4'
    operation = 'abcOperation'
    event_time = 123

    extra_value_1 = 'abc'

    extra_key_2 = 'key2'
    extra_value_2 = 123

    audit_event = {
      'serviceName': 'otherName',
      'USERNAME': username,
      'impersonator': impersonator,
      'ipAddress': ip_address,
      'operation': operation,
      'eventTime': event_time,
      'allowed': "True",
      'extra_key': extra_value_1,
      extra_key_2: extra_value_2
    }

    log_parser_model = {
      'USERNAME': {
        'fieldName': 'username'
      },
      'extra_key': {
        'fieldName': 'extraKey'
      }
    }
    result = parse_generic_audit_event(service_type, service_name, audit_event,
        log_parser_model)

    result = result[0]
    assert_equal(service_type, result['serviceType'])
    assert_equal(service_name, result['serviceName'])
    assert_equal(username, result['username']) # userName -> username
    assert_equal(impersonator, result['impersonator'])
    assert_equal(ip_address, result['ipAddress'])
    assert_equal(operation, result['operation'])
    assert_equal(event_time, result['eventTime'])
    assert_equal(True, result['allowed'])

    assert_equal(extra_value_1, result['extraValues']['extraKey'])
    assert_equal(str(extra_value_2), result['extraValues'][extra_key_2])

    audit_event['allowed'] = "False"
    result = parse_generic_audit_event(service_type, service_name, audit_event,
        log_parser_model)
    result = result[0]
    assert_equal(False, result['allowed'])
