# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

import ConfigParser
import os
import shutil
import StringIO

from cmf.audit.navigator_conf import (AUDIT_CONFIG_FILENAME,
    METADATA_CONFIG_FILENAME, AuditConfig, MetadataConfig)
from cmf.audit.navigator_logs import NoneNavigatorLogFile, NonexistentNavigatorLogFile
from cmf.audit.util import make_audit_plugin, make_metadata_plugin

from nose.tools import assert_equal, assert_false, assert_true

ROLE_CONFIG_DIR = "/tmp/audit_role_config_dir"
AUDIT_ROLE_CONFIG_DIR = "/tmp/audit_config_dir"
TP_ROLE_CONFIG_DIR = "/tmp/tp_config_dir"
AUDIT_LOG_DIR = "/tmp/audit_log"

class TestAuditPlugin(object):
  """
  Class that sets up and cleans up state for all the plugin tests.
  Contains test that are role insensitive.
  """

  def __init__(self,
               service_name = "TestService-1",
               service_type = "TestService",
               role_name = "TestRoleName",
               role_type = "TestRoleType"):
    self._service_name = service_name
    self._service_type = service_type
    self._role_name = role_name
    self._role_type = role_type

  def get_role_name(self):
    return self._role_name

  def setup(self):
    """
    Create audit log, metrics and role config directories
    """
    shutil.rmtree(ROLE_CONFIG_DIR, True)
    shutil.rmtree(AUDIT_LOG_DIR, True)
    shutil.rmtree(AUDIT_ROLE_CONFIG_DIR, True)
    shutil.rmtree(TP_ROLE_CONFIG_DIR, True)
    os.makedirs(ROLE_CONFIG_DIR)
    os.makedirs(AUDIT_LOG_DIR)
    os.makedirs(AUDIT_ROLE_CONFIG_DIR)
    os.makedirs(TP_ROLE_CONFIG_DIR)
    # Write a navigator.client.properties file.
    conf_file_path = os.path.join(ROLE_CONFIG_DIR, AUDIT_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "navigator.server.url=http://nightly-1.ent.cloudera.com:7186"
      print >> conf_file, "telemetry_publisher.server.url=http://nightly-1.ent.cloudera.com:10110"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "audit_event_log_dir=" + AUDIT_LOG_DIR
      print >> conf_file, "navigator.audit_log_max_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

    metadata_conf_file = os.path.join(ROLE_CONFIG_DIR, METADATA_CONFIG_FILENAME)
    try:
      conf_file = open(metadata_conf_file, 'w')
      print >> conf_file, "navigator.server.url=http://host00001:7187"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.client.clusterName=cluster"
      print >> conf_file, "navigator.server.timeout_seconds=120"
      print >> conf_file, "telemetry_publisher.server.url=http://host00010:7187"
      print >> conf_file, "navigator.pii.masking.enabled=true"
      print >> conf_file, "lineage_event_log_dir=" + AUDIT_LOG_DIR
    finally:
      if conf_file is not None:
        conf_file.close()

    metrics_file_path = os.path.join(AUDIT_LOG_DIR, "metrics")
    metrics_file = None
    try:
      metrics_file = open(metrics_file_path, "w")
      print >> metrics_file, "audit_plugin_parse_errors=2"
      print >> metrics_file, "audit_plugin_filtered=478"
      print >> metrics_file, "audit_plugin_coalesced=13"
      print >> metrics_file, "audit_plugin_exceptions=0"
      print >> metrics_file, "audit_plugin_events=11"
    finally:
      if metrics_file is not None:
        metrics_file.close()

    # Write a navigator.client.properties file.
    conf_file_path = os.path.join(AUDIT_ROLE_CONFIG_DIR, AUDIT_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "navigator.server.url=http://nightly-1.ent.cloudera.com:7186"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "audit_event_log_dir=" + AUDIT_LOG_DIR
      print >> conf_file, "navigator.audit_log_max_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

    # Write a navigator.client.properties file.
    conf_file_path = os.path.join(TP_ROLE_CONFIG_DIR, AUDIT_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "telemetry_publisher.server.url=http://nightly-1.ent.cloudera.com:10110"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "audit_event_log_dir=" + AUDIT_LOG_DIR
      print >> conf_file, "navigator.audit_log_max_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

  def teardown(self):
    """
    Cleanup the configuration file.
    """
    shutil.rmtree(ROLE_CONFIG_DIR)
    shutil.rmtree(AUDIT_LOG_DIR)


class TestGenericAuditPlugin(TestAuditPlugin):
  def test_generic_audit_plugin_refresh(self):
    """
    Test if generic audit plugin's refresh method picks up all the properties
    for generic plugin configuration.
    """
    plugin = make_audit_plugin("generic-PROCESS", {'conf_dir': ROLE_CONFIG_DIR})

    refresh_calls = []
    def mock_refresh_audit_thread(self):
      refresh_calls.append(self)

    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()

    assert_equal(plugin.nav_conf.log_dir, AUDIT_LOG_DIR)
    assert_equal(1, len(refresh_calls))

    assert_equal(plugin.get_service_name(), self._service_name)
    assert_equal(plugin.get_role_type(), self._role_type)
    assert_equal(plugin.get_ipc_request_name(), "writeGenericAuditEvents")
    assert_true(plugin.nav_conf.has_telemetry_publisher_url())

    from cmf.audit.generic_audit_parser import parse_generic_audit_event
    assert_true(plugin.get_log_entry_parser() is parse_generic_audit_event)

    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)

    for pipeline in pipelines:
      pipeline_name = pipeline.get_name()
      if pipeline_name and pipeline_name != "TP":
        assert_equal(pipeline._get_log_prefix(self.get_role_name()),
                     self.get_role_name() + "-" + pipeline_name)
      else:
        assert_equal(pipeline._get_log_prefix(self.get_role_name()),
                     self.get_role_name())
      assert_false(pipeline.get_nav_logs_dir() is None)
      assert_true(type(pipeline.get_current_nav_log()) is NoneNavigatorLogFile)

      file_name, offset = pipeline.get_wal().get_file_info()
      assert_true(file_name is None)
      assert_true(offset is None)
      pipeline.switch_log("non-existing-file")
      assert_true(type(pipeline.get_current_nav_log()) is NonexistentNavigatorLogFile)

  def test_disabled_generic_audit_plugin_refresh(self):
    """
    Test if disabled generic audit plugin's refresh method is skipped.
    """
    plugin = make_audit_plugin("generic-PROCESS")

    refresh_calls = []
    def mock_refresh_audit_thread(self):
      refresh_calls.append(self)

    plugin.refresh(ROLE_CONFIG_DIR + 'x', refresh_thread=mock_refresh_audit_thread)

    assert_false(plugin.is_nav_enabled())
    assert_equal(0, len(refresh_calls))

  def test_metadata_plugin(self):
    """
    Test methods in metadata plugin to confirm thing work correctly.
    """
    plugin = make_metadata_plugin("generic-PROCESS")

    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=None)
    plugin.immediate_refresh()

    cluster_name = "cluster-1"
    service_name = "service-1"
    assert_equal(plugin.get_api_url(cluster_name, service_name, None, True),
                 '/api/v1/operations/%s/%s' % (cluster_name, service_name))

  def test_metadata_plugin_without_telemetry_pub(self):
    """
    Test that correct api url is returned when telemetry publisher is absent.
    """
    conf_str = '[navigator]\n' + \
    'navigator.server.url=http://nms_host:7186'

    conf_fd = StringIO.StringIO(conf_str)
    config = ConfigParser.SafeConfigParser()
    config.readfp(conf_fd)
    options = {}
    for key_value in config.items("navigator"):
      options["navigator." + key_value[0]] = key_value[1]
    conf = MetadataConfig(options)

    plugin = make_metadata_plugin("hbase-MASTER")
    plugin.nav_conf = conf

    assert_false(plugin.nav_conf.has_telemetry_publisher_url())
    assert_equal(plugin.get_api_url("cluster", "service"), "/api/v5/metadata/cluster/service")

  def test_audit_plugin_with_telemetry_pub(self):
    """
    Test that correct api url is returned when telemetry publisher is present.
    """
    conf_str = '[navigator]\n' + \
    'navigator.server.url=http://nms_host:7186\n' + \
    'telemetry_publisher.server.url=http://tp_host:7187'

    conf_fd = StringIO.StringIO(conf_str)
    config = ConfigParser.SafeConfigParser()
    config.readfp(conf_fd)
    options = {}
    for key_value in config.items("navigator"):
      options["navigator." + key_value[0]] = key_value[1]
    conf = AuditConfig(options)

    plugin = make_audit_plugin("hive-HIVESERVER2")
    plugin.nav_conf = conf

    assert_true(plugin.nav_conf.has_telemetry_publisher_url())
    assert_equal(plugin.get_api_url("cluster", "service"), "/api/v1/audits/cluster/service")

  def test_audit_plugin_with_tp_on_and_off(self):
    """
    Test that checks plugin and it pipelines with below cases:
    1) Both TP and Audit server enabled
    2) Just Audit server enabled
    3) Just TP enabled
    4) Nothing enabled
    """

    plugin = make_audit_plugin("hive-HIVESERVER2")
    refresh_calls = []
    def mock_refresh_audit_thread(self):
      refresh_calls.append(self)

    # start the plugin
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
    # restart the plugin, pipeline should be as before
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)

    # restart the plugin with audit config, pipeline should be only audits
    plugin.refresh(AUDIT_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 3)
    # restart the plugin with both audit and tp config, pipeline should be both
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)
    # restart the plugin with tp config, pipeline should be only tp
    plugin.refresh(TP_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 3)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)

    # restart the plugin with tp config, pipeline should be only tp
    plugin.refresh(TP_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 3)
    # restart the plugin with both audit and tp config, pipeline should be both
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)
    # restart the plugin with audit config, pipeline should be only audit
    plugin.refresh(AUDIT_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 3)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)

    # restart the plugin with audit config, pipeline should be only audits
    plugin.refresh(AUDIT_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 3)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
    # restart the plugin with tp config, pipeline should be only tp
    plugin.refresh(TP_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 3)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
    # restart the plugin with both audit and tp config, pipeline should be both
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 6)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
