# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
import os
import shutil
import cmf.audit

from cmf.audit import AUDIT_CONFIG_FILENAME
from cmf.audit.navigator_conf import DEFAULT_MIN_BATCH_SIZE, DEFAULT_MAX_REQUEST_SIZE
from cmf.audit.navigator_sender import Result, AuditSender
from cmf.audit.navigator_thread import AuditThread
from cmf.audit.util import make_audit_plugin
from nose.tools import assert_equal, nottest
from threading import Event

_AUDIT_LOG_DIR = "/tmp/audit_log"
_AUDIT_LOG_DIR_2 = "/tmp/audit_log2"
_ROLE_CONFIG_DIR = "/tmp/nav_client_config_dir"
_IMPALA_AUDIT_FILE_NAME = "impala_audit_event_log_1.0-1377798507857"
_IMPALA_AUDIT_LOG_TEXT = """{"1377775857227":{"query_id":"84bd714e4ef1fa3:75ff54a3b1824295","session_id":"2d4c159702aeebe2:cb5377ebedf6168b","start_time":"2013-08-29 11:30:56.897841000","authorization_failure":false,"status":"","user":"impalauser","impersonator":null,"statement_type":"CREATE_DATABASE","network_address":"::ffff:127.0.0.1:34009","sql_statement":"create DATABASE impalaQueryDB_1377801039","catalog_objects":[{"name":"impalaQueryDB_1377801039","object_type":"DATABASE","privilege":"CREATE"}]}}
{"1377775857472":{"query_id":"70413d07e1ee7ec6:5d1b5f76aee04183","session_id":"2d4c159702aeebe2:cb5377ebedf6168b","start_time":"2013-08-29 11:30:57.441235000","authorization_failure":false,"status":"","user":"impalauser","impersonator":null,"statement_type":"USE","network_address":"::ffff:127.0.0.1:34009","sql_statement":"use impalaQueryDB_1377801039","catalog_objects":[{"name":"impalaQueryDB_1377801039","object_type":"DATABASE","privilege":"ANY"}]}}
{"1377775857516":{"query_id":"d04daaf9e488e953:5cb8bdc461eb9895","session_id":"2d4c159702aeebe2:cb5377ebedf6168b","start_time":"2013-08-29 11:30:57.488402000","authorization_failure":false,"status":"","user":"impalauser","impersonator":null,"statement_type":"DROP_TABLE","network_address":"::ffff:127.0.0.1:34009","sql_statement":"drop TABLE IF EXISTS testaudittable1","catalog_objects":[]}}"""

_NN_AUDIT_FILE_NAME = "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b-HdfsAuditLogger-1385512800584"
_NN_AUDIT_LOG_TEXT = """{"allowed":true,"serviceName":"HDFS-1","username":"hbase","src":"/hbase/hbase.id","eventTime":1385512859079,"ipAddress":"10.20.187.50","operation":"getfileinfo","dest":null,"permissions":null,"impersonator":null,"delegationTokenId":null}
{"allowed":true,"serviceName":"HDFS-1","username":"hbase","src":"/hbase/hbase.version","eventTime":1385512859081,"ipAddress":"10.20.187.50","operation":"getfileinfo","dest":null,"permissions":null,"impersonator":null,"delegationTokenId":null}
{"allowed":true,"serviceName":"HDFS-1","username":"cloudera-scm","src":"/tmp/.cloudera_health_monitoring_canary_files","eventTime":1385512859245,"ipAddress":"10.20.187.50","operation":"getfileinfo","dest":null,"permissions":null,"impersonator":null,"delegationTokenId":null}
{"allowed":true,"serviceName":"HDFS-1","username":"cloudera-scm","src":"/tmp/.cloudera_health_monitoring_canary_files/.canary_file_2013_11_26-16_40_59","eventTime":1385512859291,"ipAddress":"10.20.187.50","operation":"create","dest":null,"permissions":"rw-rw-rw-","impersonator":null,"delegationTokenId":null}
{"allowed":true,"serviceName":"HDFS-1","username":"cloudera-scm","src":"/tmp/.cloudera_health_monitoring_canary_files/.canary_file_2013_11_26-16_40_59","eventTime":1385512859376,"ipAddress":"10.20.187.50","operation":"open","dest":null,"permissions":null,"impersonator":null,"delegationTokenId":null}"""

_MASTER_AUDIT_FILE_NAME="HBASE-1-MASTER-6fd97b11220b3ce199c6826a1abf4b61-MasterAuditCoProcessor-1385581498660"
_MASTER_AUDIT_LOG_TEXT="""{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385581585702,"ipAddress":"10.20.187.50","tableName":"gov_test","family":null,"qualifier":null,"operation":"createTable","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385581630088,"ipAddress":"10.20.187.50","tableName":"gov_test","family":null,"qualifier":null,"operation":"disableTable","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385581632201,"ipAddress":"10.20.187.50","tableName":"gov_test","family":null,"qualifier":null,"operation":"deleteTable","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385581650658,"ipAddress":"10.20.187.50","tableName":"gov_test1","family":null,"qualifier":null,"operation":"createTable","impersonator":null}"""

_RS_AUDIT_FILE_NAME="HBASE-1-REGIONSERVER-768eeea3e4d63d7071313c0f228310be-RegionAuditCoProcessor-1385602765230"
_RS_AUDIT_LOG_TEXT="""{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385602867073,"ipAddress":"10.20.187.50","tableName":"-ROOT-","family":"info","qualifier":null,"operation":"getClosestRowBefore","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385602867097,"ipAddress":"10.20.187.50","tableName":".META.","family":"info","qualifier":null,"operation":"get","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385602867097,"ipAddress":"10.20.187.50","tableName":".META.","family":"info","qualifier":null,"operation":"getClosestRowBefore","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385602867272,"ipAddress":"10.20.187.50","tableName":"gov_test","family":"cf","qualifier":null,"operation":"scannerOpen","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385602902713,"ipAddress":"10.20.187.50","tableName":"-ROOT-","family":"info","qualifier":null,"operation":"get","impersonator":null}
{"allowed":true,"serviceName":"HBASE-1","username":"UNKNOWN","eventTime":1385602902713,"ipAddress":"10.20.187.50","tableName":"-ROOT-","family":"info","qualifier":null,"operation":"getClosestRowBefore","impersonator":null}"""

_HS2_AUDIT_FILE_NAME="HIVE-HIVESERVER2-1-HiveExecHookContext-1386288084237"
_HS2_AUDIT_LOG_TEXT="""{"serviceName":"HIVE-1","username":"systest","impersonator":null,"ipAddress":"10.20.187.50","operation":"QUERY","eventTime":1386288397274,"operationText":"INSERT OVERWRITE LOCAL DIRECTORY '/tmp/hive_workflow_test' SELECT foo.rating, count(DISTINCT foo.userid) FROM ratings_int foo GROUP BY foo.rating","allowed":true,"databaseName":"default","tableName":"ratings_int","resourcePath":"/user/hive/warehouse/ratings_int/dt=2013-01-31/country=DK","objectType":"PARTITION"}
{"serviceName":"HIVE-1","username":"systest","impersonator":null,"ipAddress":"10.20.187.50","operation":"QUERY","eventTime":1386288397274,"operationText":"INSERT OVERWRITE LOCAL DIRECTORY '/tmp/hive_workflow_test' SELECT foo.rating, count(DISTINCT foo.userid) FROM ratings_int foo GROUP BY foo.rating","allowed":true,"databaseName":"","tableName":"","resourcePath":"/tmp/hive_workflow_test","objectType":"LOCAL_DIR"}
{"serviceName":"HIVE-1","username":"systest","impersonator":null,"ipAddress":"10.20.187.50","operation":"DROPTABLE","eventTime":1386288404236,"operationText":"DROP TABLE ratings_stg","allowed":true,"databaseName":"default","tableName":"ratings_stg","resourcePath":"/user/hive/warehouse/ratings1.dat","objectType":"TABLE"}
{"serviceName":"HIVE-1","username":"systest","impersonator":null,"ipAddress":"10.20.187.50","operation":"DROPTABLE","eventTime":1386288407237,"operationText":"DROP TABLE ratings_int","allowed":true,"databaseName":"default","tableName":"ratings_int","resourcePath":"/user/hive/warehouse/ratings_int","objectType":"TABLE"}
{"serviceName":"HIVE-1","username":"systest","impersonator":null,"ipAddress":"10.20.187.50","operation":"DROPTABLE","eventTime":1386288409765,"operationText":"DROP TABLE ratings_sum","allowed":true,"databaseName":"default","tableName":"ratings_sum","resourcePath":"/user/hive/warehouse/ratings_sum","objectType":"TABLE"}"""

_BEESWAX_AUDIT_FILE_NAME="HUE-1-BEESWAX_SERVER-831ade23fdc097d831bb9caf281c14b0-HiveMetaStoreEventListener-1386882731317"
_BEESWAX_AUDIT_LOG_TEXT="""{"serviceName":"HUE-1","username":"admin","impersonator":null,"ipAddress":"unknown","operation":"QUERY","eventTime":1386882732123,"operationText":"select * from sample_07","allowed":true,"databaseName":"default","tableName":"sample_07","resourcePath":"/user/hive/warehouse/sample_07","objectType":"TABLE"}
{"serviceName":"HUE-1","username":"admin","impersonator":null,"ipAddress":"unknown","operation":"QUERY","eventTime":1386882732331,"operationText":"select * from sample_07","allowed":true,"databaseName":"","tableName":"","resourcePath":"/tmp/hive-beeswax-admin/hive_2013-12-12_13-12-11_324_805092464982560610-1/-mr-10000","objectType":"DFS_DIR"}
{"serviceName":"HUE-1","username":"admin","impersonator":null,"ipAddress":"unknown","operation":"QUERY","eventTime":1386882784259,"operationText":"select * from sample_07 where code = '11-3071'","allowed":true,"databaseName":"default","tableName":"sample_07","resourcePath":"/user/hive/warehouse/sample_07","objectType":"TABLE"}
{"serviceName":"HUE-1","username":"admin","impersonator":null,"ipAddress":"unknown","operation":"QUERY","eventTime":1386882784259,"operationText":"select * from sample_07 where code = '11-3071'","allowed":true,"databaseName":"","tableName":"","resourcePath":"/tmp/hive-beeswax-admin/hive_2013-12-12_13-12-48_899_24184094469501974-1/-mr-10000","objectType":"DFS_DIR"}"""

class MockAuditSender(AuditSender):
  """
  Mock audit sender object that we will pass to AuditThread for it to send
  events. Lets us validate events sent by audit thread.
  """
  def __init__(self, event, success_count = 1, failure_count = 0):
    """
    Mock sender to send audit event to audit server. It sets the "event"
    when "send_nav_log_entries" method is called. If "failure_count" is set,
    then it fails the request to send "failure_count" number of times
    before succeeding.
    """
    self._audit_events = []
    self._plugin = None
    self._event = event
    self._failure_count = failure_count
    self._success_count = success_count

  def send_nav_log_entries(self, plugin, log_entry_batch):
    """
    Store the arguments and make them available for correctness checking
    """
    self._plugin = plugin
    audit_events = [audit_event for (_, _, audit_event) in log_entry_batch]

    # Invoke the audit event parser to makes sure it can parse events correctly
    self._get_parsed_entries(plugin, log_entry_batch)

    if self._failure_count > 0:
      self._failure_count -= 1
      return Result.FAILED
    elif self._success_count > 1:
      self._audit_events.extend(audit_events)
      self._success_count -= 1
    else:
      self._audit_events.extend(audit_events)
      self._event.set()

    return Result.SUCCESS

  def get_event_count(self):
    return len(self._audit_events)

  def get_event_parser(self):
    if self._plugin is None:
      return None
    else:
      return self._plugin.get_log_entry_parser()

  def get_success_count(self):
    return self._success_count

  def get_failure_count(self):
    return self._failure_count

  def get_service_name(self):
    if self._plugin is None:
      return None
    else:
      return self._plugin.get_service_name()

  def get_service_type(self):
    if self._plugin is None:
      return None
    else:
      return self._plugin.get_service_type()

class TestAuditThread(object):
  """
  Test that main audit thread can process events for all role types.
  """
  '''
  def setup(self):
    """
    Setup audit logs and audit client configuration.
    """
    shutil.rmtree(_ROLE_CONFIG_DIR, True)
    shutil.rmtree(_AUDIT_LOG_DIR, True)
    shutil.rmtree(_AUDIT_LOG_DIR_2, True)
    os.makedirs(_ROLE_CONFIG_DIR)
    os.makedirs(_AUDIT_LOG_DIR)
    os.makedirs(_AUDIT_LOG_DIR_2)
    # stop the main audit thread
    cmf.audit.audit_thread.stop()

  def teardown(self):
    """
    Cleanup audit logs and client configuration.
    """
    shutil.rmtree(_ROLE_CONFIG_DIR)
    shutil.rmtree(_AUDIT_LOG_DIR)
    shutil.rmtree(_AUDIT_LOG_DIR_2)
  '''
  def _create_audit_log_file(self,
                             file_name,
                             audit_dir = _AUDIT_LOG_DIR,
                             content = "Test Audit Log Line"):
    """
    Creates a new audit log file and returns the path to the file.
    """
    audit_file_path = os.path.join(audit_dir, file_name)
    audit_file = open(audit_file_path, "w")
    print >> audit_file, content
    audit_file.close()

    return audit_file_path

  def _create_nav_client_config(self, service_name, service_type, role_name,
      role_type, batch_size=DEFAULT_MIN_BATCH_SIZE,
      max_request_size=DEFAULT_MAX_REQUEST_SIZE, audit_log_dir=_AUDIT_LOG_DIR):
    """
    Write a navigator.client.properties file.
    """
    conf_file_path = os.path.join(_ROLE_CONFIG_DIR, AUDIT_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "navigator.server.url=http://nightly-1.ent.cloudera.com:7186"
      print >> conf_file, "navigator.client.serviceName=" + service_name
      print >> conf_file, "navigator.client.serviceType=" + service_type
      print >> conf_file, "navigator.client.roleName=" + role_name
      print >> conf_file, "navigator.client.roleType=" + role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "audit_event_log_dir=" + audit_log_dir
      print >> conf_file, "navigator.batch.timeout=0"
      print >> conf_file, "navigator.batch.min_size=%d" % (batch_size,)
      print >> conf_file, "navigator.client.max_request_size=%d" % (max_request_size,)
    finally:
      if conf_file is not None:
        conf_file.close()

    return conf_file_path

  def refresh_audit_thread(self, audit_thread, audit_sender):
    audit_thread.refresh_nav_plugin(audit_sender)

  def _check_audit_events(self, service_name, audit_plugin, audit_file_name,
                          audit_file_text, num_event,
                          event_parser = "cmf.audit.nop_audit_parser",
                          expected_parser_name = "parse_audit_event",
                          success_count = 1,
                          failure_count = 0):
    """
    Helper method to contain all the common code for tests.
    """
    event = Event()
    audit_sender = MockAuditSender(event, success_count, failure_count)
    test_audit_thread = AuditThread(0, "TestAuditThread")
    audit_plugin.refresh(_ROLE_CONFIG_DIR,
                         refresh_thread= lambda _ : None)
    audit_plugin.immediate_refresh()
    test_audit_thread.add_navigator_plugin(audit_plugin)
    test_audit_thread.refresh_nav_plugin(audit_plugin, audit_sender)

    # Now create a audit log file and put few events in it
    # Then check if the events are received correctly
    self._create_audit_log_file(audit_file_name, content = audit_file_text)

    event.wait(60) # Wait for event to be sent
    test_audit_thread.stop()
    assert_equal(audit_sender.get_service_name(), service_name)
    assert_equal(audit_sender.get_event_count(), num_event)
    assert_equal(audit_sender.get_event_parser().func_name, expected_parser_name)
    assert_equal(audit_sender.get_event_parser().__module__, event_parser)
    assert_equal(audit_sender.get_success_count(), 1)
    assert_equal(audit_sender.get_failure_count(), 0)

  @nottest
  def test_impalad_audit_events(self):
    """
    Test if audit events from impalad can be parsed successfully.
    """
    service_name = "IMPALA-1"
    self._create_nav_client_config(service_name, "IMPALA",
      "IMPALA-1-IMPALAD-abee5a758942d6c689c7a5e7dd2da179",
      "IMPALAD")

    self._check_audit_events(service_name,
                             make_audit_plugin("impala-IMPALAD"),
                             _IMPALA_AUDIT_FILE_NAME,
                             _IMPALA_AUDIT_LOG_TEXT,
                             3,
                             "cmf.audit.impala.impalad_audit_parser",
                             "parse_impalad_audit_event")

  @nottest
  def test_namenode_audit_events(self):
    """
    Test if audit events from namenode can be parsed successfully.
    """
    service_name = "HDFS-1"
    self._create_nav_client_config(service_name, "HDFS",
      "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b",
      "NAMENODE")

    self._check_audit_events(service_name,
                             make_audit_plugin("hdfs-NAMENODE"),
                             _NN_AUDIT_FILE_NAME,
                             _NN_AUDIT_LOG_TEXT,
                             5)

  @nottest
  def test_hbase_master_audit_events(self):
    """
    Test if audit events from hbase master can be parsed successfully.
    """
    service_name = "HBASE-1"
    self._create_nav_client_config(service_name, "HBASE",
      "HBASE-1-MASTER-6fd97b11220b3ce199c6826a1abf4b61",
      "MASTER")

    self._check_audit_events(service_name,
                             make_audit_plugin("hbase-MASTER"),
                             _MASTER_AUDIT_FILE_NAME,
                             _MASTER_AUDIT_LOG_TEXT,
                             4)

  @nottest
  def test_hbase_regionserver_audit_events(self):
    """
    Test if audit events from hbase regionserver can be parsed successfully.
    """
    service_name = "HBASE-1"
    self._create_nav_client_config(service_name, "HBASE",
      "HBASE-1-REGIONSERVER-768eeea3e4d63d7071313c0f228310be",
      "REGIONSERVER")

    self._check_audit_events(service_name,
                             make_audit_plugin("hbase-REGIONSERVER"),
                             _RS_AUDIT_FILE_NAME,
                             _RS_AUDIT_LOG_TEXT,
                             6)

  @nottest
  def test_hive_hs2_audit_events(self):
    """
    Test if audit events from HiveServer2 can be parsed successfully.
    """
    service_name = "HIVE-1"
    self._create_nav_client_config(service_name, "HIVE",
      "HIVE-HIVESERVER2-1",
      "HIVESERVER2")

    self._check_audit_events(service_name,
                             make_audit_plugin("hive-HIVESERVER2"),
                             _HS2_AUDIT_FILE_NAME,
                             _HS2_AUDIT_LOG_TEXT,
                             5)

  @nottest
  def test_hue_beeswax_audit_events(self):
    """
    Test if audit events from Beeswax server can be parsed successfully.
    """
    service_name = "HUE-1"
    self._create_nav_client_config(service_name, "HUE",
      "HUE-1-BEESWAX_SERVER-831ade23fdc097d831bb9caf281c14b0",
      "BEESWAX_SERVER")

    self._check_audit_events(service_name,
                             make_audit_plugin("hue-BEESWAX_SERVER"),
                             _BEESWAX_AUDIT_FILE_NAME,
                             _BEESWAX_AUDIT_LOG_TEXT,
                             4)
  @nottest
  def test_event_retry(self):
    """
    Test if the sender fails to send message, we retry those till we succeed.
    This test makes sure we don't drop events in case of any failure when
    transmitting them.
    """
    service_name = "HDFS-1"
    self._create_nav_client_config(service_name, "HDFS",
      "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b",
      "NAMENODE")

    self._check_audit_events(service_name,
                             make_audit_plugin("hdfs-NAMENODE"),
                             _NN_AUDIT_FILE_NAME,
                             _NN_AUDIT_LOG_TEXT,
                             5,
                             failure_count = 2)
  @nottest
  def test_max_request_size(self):
    """
    Test if the number of audit events exceed max request size, we break it into
    multiple send requests.
    """
    service_name = "HDFS-1"
    self._create_nav_client_config(service_name, "HDFS",
      "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b",
      "NAMENODE",
      max_request_size = 200)

    self._check_audit_events(service_name,
                             make_audit_plugin("hdfs-NAMENODE"),
                             _NN_AUDIT_FILE_NAME,
                             _NN_AUDIT_LOG_TEXT,
                             5,
                             success_count = 5)
  @nottest
  def test_batch_size(self):
    """
    Test if number of events are more than the batch size, we break it into
    multiple requests.
    """
    service_name = "HDFS-1"
    self._create_nav_client_config(service_name, "HDFS",
      "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b",
      "NAMENODE",
      batch_size = 3)

    self._check_audit_events(service_name,
                             make_audit_plugin("hdfs-NAMENODE"),
                             _NN_AUDIT_FILE_NAME,
                             _NN_AUDIT_LOG_TEXT,
                             5,
                             success_count = 2)
  @nottest
  def test_audit_dir_move(self):
    """
    Test when audit directory moves, we pick up new directory and read logs
    from the new directory.
    OPSAPS-17815 reported this issue, so created test to make sure we don't
    break it in future.
    """
    service_name = "HDFS-1"
    conf_file_path = self._create_nav_client_config(service_name, "HDFS",
      "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b",
      "NAMENODE")

    event = Event()
    audit_plugin = make_audit_plugin("hdfs-NAMENODE")
    audit_sender = MockAuditSender(event, 1, 0)
    test_audit_thread = AuditThread(0, "TestAuditThread")

    audit_plugin.refresh(_ROLE_CONFIG_DIR,
                         refresh_thread= lambda _ : None)
    audit_plugin.immediate_refresh()
    test_audit_thread.add_navigator_plugin(audit_plugin)
    test_audit_thread.refresh_nav_plugin(audit_plugin, audit_sender)

    # Now create a audit log file and put few events in it
    self._create_audit_log_file(_NN_AUDIT_FILE_NAME, content = _NN_AUDIT_LOG_TEXT)

    event.wait(60) # Wait for event to be sent
    event.clear() # reset event so that we can wait again

    # Check that it worked
    assert_equal(audit_sender.get_service_name(), service_name)
    assert_equal(audit_sender.get_event_count(), 5)
    assert_equal(audit_sender.get_success_count(), 1)
    assert_equal(audit_sender.get_failure_count(), 0)

    # Now do this again with a new audit log directory
    os.remove(conf_file_path)
    self._create_nav_client_config(service_name, "HDFS",
      "HDFS-1-NAMENODE-8289bf63ac31fd75423fb00cf25e8e3b",
      "NAMENODE",
      audit_log_dir = _AUDIT_LOG_DIR_2)

    audit_plugin.refresh(_ROLE_CONFIG_DIR,
                         refresh_thread= lambda _ : None)
    audit_plugin.immediate_refresh()
    test_audit_thread.add_navigator_plugin(audit_plugin)
    test_audit_thread.refresh_nav_plugin(audit_plugin, audit_sender)

    # Now create a audit log file and put few events in it
    self._create_audit_log_file(_NN_AUDIT_FILE_NAME,
                                audit_dir = _AUDIT_LOG_DIR_2,
                                content = _NN_AUDIT_LOG_TEXT)

    event.wait(60) # Wait for event to be sent
    test_audit_thread.stop()

    assert_equal(audit_sender.get_event_count(), 10)
    assert_equal(audit_sender.get_success_count(), 1)
    assert_equal(audit_sender.get_failure_count(), 0)
