# Copyright (c) 2017 Cloudera, Inc. All rights reserved.

import cmf.audit
import cmf.audit.util
import os
import shutil
import tempfile

from cmf.audit.navigator_thread import MetadataThread, NavigatorThread
from nose.tools import assert_true
from zipfile import ZipFile
from StringIO import StringIO

class TestHeartbeatCallback(object):

  def _create_zip(self, files=None):
    data = StringIO()
    zipfile = ZipFile(data, "w")
    if files:
      for fname, content in files.iteritems():
        zipfile.writestr(fname, content)
    zipfile.close()
    return data.getvalue()

  def _create_config(self, service_name):
    alt_name = "spark-conf"
    nav_config = os.path.join(alt_name, cmf.audit.METADATA_CONFIG_FILENAME)
    conf_data = self._create_zip({ nav_config : "unused" })

    conf_dir = os.path.join(self._tmp_dir, service_name)
    os.mkdir(conf_dir)
    open(os.path.join(conf_dir, cmf.audit.METADATA_CONFIG_FILENAME), "w").write("unused")
    return {
      'alt_name'  : alt_name,
      'data'      : conf_data,
      'path'      : conf_dir,
    }

  def setup(self):
    self._tmp_dir = tempfile.mkdtemp()
    cmf.audit.metadata_thread = MetadataThread(0, "TestHeartbeatCallbackThread")

  def teardown(self):
    # Remove the thread from the static list to avoid it interfering with other tests.
    NavigatorThread.THREADS.remove(cmf.audit.metadata_thread)
    shutil.rmtree(self._tmp_dir)

  def test_multiple_services(self):
    service1 = self._create_config("service1")
    service2 = self._create_config("service2")
    response = {
      'client_configs' : [ service1, service2 ]
    }

    # Call the monitor and verify both services were registered.
    cmf.audit.util.watch_gateway_roles(None, response)
    assert_true(cmf.audit.metadata_thread.is_plugin_already_added("service1"))
    assert_true(cmf.audit.metadata_thread.is_plugin_already_added("service2"))
