# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
import os
import shutil

from cmf.audit import METADATA_CONFIG_FILENAME
from cmf.audit.navigator_logs import NoneNavigatorLogFile, NonexistentNavigatorLogFile
from cmf.audit.util import make_metadata_plugin

from nose.tools import assert_equal, assert_false, assert_true

ROLE_CONFIG_DIR = "/tmp/metadata_role_config_dir"
METADATA_LOG_DIR = "/tmp/metadata_log"
METADATA_ROLE_CONFIG_DIR = "/tmp/audit_config_dir"
TP_ROLE_CONFIG_DIR = "/tmp/tp_config_dir"

class TestMetadataPlugin(object):
  """
  Class that sets up and cleans up state for all the plugin tests.
  Contains test that are role insensitive.
  """

  def __init__(self,
               service_name = "TestService-1",
               service_type = "TestService",
               role_name = "TestRoleName",
               role_type = "TestRoleType"):
    self._service_name = service_name
    self._service_type = service_type
    self._role_name = role_name
    self._role_type = role_type

  def get_role_name(self):
    return self._role_name

  def setup(self):
    """
    Create metadata log, metrics and role config directories
    """
    shutil.rmtree(ROLE_CONFIG_DIR, True)
    shutil.rmtree(METADATA_LOG_DIR, True)
    shutil.rmtree(METADATA_ROLE_CONFIG_DIR, True)
    shutil.rmtree(TP_ROLE_CONFIG_DIR, True)
    os.makedirs(ROLE_CONFIG_DIR)
    os.makedirs(METADATA_LOG_DIR)
    os.makedirs(METADATA_ROLE_CONFIG_DIR)
    os.makedirs(TP_ROLE_CONFIG_DIR)

    # Write a navigator.client.properties file.
    conf_file_path = os.path.join(ROLE_CONFIG_DIR, METADATA_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "navigator.server.url=http://nightly-1.ent.cloudera.com:7186"
      print >> conf_file, "telemetry_publisher.server.url=http://nightly-1.ent.cloudera.com:10110"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "lineage_event_log_dir=" + METADATA_LOG_DIR
      print >> conf_file, "navigator.audit_log_max_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

    metrics_file_path = os.path.join(METADATA_LOG_DIR, "metrics")
    metrics_file = None
    try:
      metrics_file = open(metrics_file_path, "w")
      print >> metrics_file, "audit_plugin_parse_errors=2"
      print >> metrics_file, "audit_plugin_filtered=478"
      print >> metrics_file, "audit_plugin_coalesced=13"
      print >> metrics_file, "audit_plugin_exceptions=0"
      print >> metrics_file, "audit_plugin_events=11"
    finally:
      if metrics_file is not None:
        metrics_file.close()

    # Write a navigator.client.properties file.
    conf_file_path = os.path.join(METADATA_ROLE_CONFIG_DIR, METADATA_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "navigator.server.url=http://nightly-1.ent.cloudera.com:7186"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "lineage_event_log_dir=" + METADATA_LOG_DIR
      print >> conf_file, "navigator.audit_log_max_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

    # Write a navigator.client.properties file.
    conf_file_path = os.path.join(TP_ROLE_CONFIG_DIR, METADATA_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "telemetry_publisher.server.url=http://nightly-1.ent.cloudera.com:10110"
      print >> conf_file, "navigator.client.serviceName=" + self._service_name
      print >> conf_file, "navigator.client.serviceType=" + self._service_type
      print >> conf_file, "navigator.client.roleName=" + self._role_name
      print >> conf_file, "navigator.client.roleType=" + self._role_type
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "lineage_event_log_dir=" + METADATA_LOG_DIR
      print >> conf_file, "navigator.audit_log_max_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

  def teardown(self):
    """
    Cleanup the configuration file.
    """
    shutil.rmtree(ROLE_CONFIG_DIR)
    shutil.rmtree(METADATA_LOG_DIR)

  def test_metadata_plugin_with_tp_on_and_off(self):
    """
    Test that checks plugin and it pipelines with below cases:
    1) Both TP and metadata server enabled
    2) Just metadata server enabled
    3) Just TP enabled
    4) Nothing enabled
    """

    plugin = make_metadata_plugin("impala-IMPALAD")
    refresh_calls = []
    def mock_refresh_audit_thread(self):
      refresh_calls.append(self)

    # start the plugin
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
    # restart the plugin, pipeline should be as before
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)

    # restart the plugin with metadata config, pipeline should be only metadata
    plugin.refresh(METADATA_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 1)
    # restart the plugin with both metadata and tp config, pipeline should be both
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)
    # restart the plugin with tp config, pipeline should be only tp
    plugin.refresh(TP_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 1)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)

    # restart the plugin with tp config, pipeline should be only tp
    plugin.refresh(TP_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 1)
    # restart the plugin with both metadata and tp config, pipeline should be both
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)
    # restart the plugin with metadata config, pipeline should be only metadata
    plugin.refresh(METADATA_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 1)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)

    # restart the plugin with metadata config, pipeline should be only metadata
    plugin.refresh(METADATA_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 1)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
    # restart the plugin with tp config, pipeline should be only tp
    plugin.refresh(TP_ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 1)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
    # restart the plugin with both metadata and tp config, pipeline should be both
    plugin.refresh(ROLE_CONFIG_DIR, refresh_thread=mock_refresh_audit_thread)
    plugin.immediate_refresh()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 2)
    # stop the plugin, pipeline should be empty
    plugin.immediate_stop()
    pipelines = plugin.get_nav_pipelines()
    assert_equal(len(pipelines), 0)
