# Copyright (c) 2017 Cloudera, Inc. All rights reserved.

import ConfigParser
import os
import shutil
import StringIO

from cmf.audit import parse_audit_configuration
from cmf.audit.navigator_conf import (AUDIT_CONFIG_FILENAME,
    DEFAULT_AUDIT_BATCH_TIMEOUT, DEFAULT_MIN_BATCH_SIZE, MetadataConfig)
from nose.tools import assert_equal, assert_true, assert_false

_ROLE_CONFIG_DIR = "/tmp/namenode"

class TestAudit(object):
  """
  This class tests methods in the audit module.
  """

  def setup(self):
    """
    Write a navigator.client.properties file.
    """
    shutil.rmtree(_ROLE_CONFIG_DIR, True)
    os.makedirs(_ROLE_CONFIG_DIR)

    conf_file_path = os.path.join(_ROLE_CONFIG_DIR, AUDIT_CONFIG_FILENAME)
    conf_file = None
    try:
      conf_file = open(conf_file_path, 'w')
      print >> conf_file, "navigator.server.url=http://nightly-1.ent.cloudera.com:7186"
      print >> conf_file, "navigator.client.serviceName=HDFS-1"
      print >> conf_file, "navigator.client.serviceType=HDFS"
      print >> conf_file, "navigator.client.roleName=HDFS-1-NAMENODE-abee5a758942d6c689c7a5e7dd2da179"
      print >> conf_file, "navigator.client.roleType=NAMENODE"
      print >> conf_file, "navigator.batch.queue_policy=DROP"
      print >> conf_file, "audit_event_log_dir=/var/log/hadoop-hdfs/audit"
      print >> conf_file, "max_audit_event_log_file_size=100"
      print >> conf_file, "navigator.client.max_num_audit_log=10"
    finally:
      if conf_file is not None:
        conf_file.close()

  def teardown(self):
    """
    Cleanup the configuration file.
    """
    shutil.rmtree(_ROLE_CONFIG_DIR)

  def test_parse_audit_configuration(self):
    """
    Test if a correct navigator.client.conf can be parsed correctly.
    """
    audit_conf = parse_audit_configuration(_ROLE_CONFIG_DIR)
    assert_equal("/var/log/hadoop-hdfs/audit", audit_conf.log_dir)
    assert_equal("http://nightly-1.ent.cloudera.com:7186", audit_conf.server_url)
    assert_equal("NAMENODE", audit_conf.role_type)
    assert_equal("HDFS-1-NAMENODE-abee5a758942d6c689c7a5e7dd2da179", audit_conf.role_name)
    assert_equal("HDFS-1", audit_conf.service_name)
    assert_equal("HDFS", audit_conf.service_type)
    assert_equal("/var/log/hadoop-hdfs/audit", audit_conf.log_dir)
    assert_equal(10, audit_conf.max_num_navigator_logs)
    assert_equal(DEFAULT_MIN_BATCH_SIZE, audit_conf.batch_min_size)
    assert_equal(DEFAULT_AUDIT_BATCH_TIMEOUT, audit_conf.batch_timeout)

  def test_server_url(self):
    """
    Test if server url is returned correctly. We prioritize telemetry publisher
    over NMS server in this release. So if telemetry publisher url is present, we
    honor that, otherwise we use the nms url.
    """
    conf_str = '[navigator]\n' + \
    'navigator.server.url=http://nms_host:7187\n' + \
    'telemetry_publisher.server.url=http://tp_host:7187'

    conf_fd = StringIO.StringIO(conf_str)
    config = ConfigParser.SafeConfigParser()
    config.readfp(conf_fd)
    options = {}
    for key_value in config.items("navigator"):
      options["navigator." + key_value[0]] = key_value[1]
    conf = MetadataConfig(options)
    assert_equal("http://tp_host:7187", conf.server_url)
    assert_true(conf.has_telemetry_publisher_url)

    conf_str = '[navigator]\n' + \
    'navigator.server.url=http://nms_host:7187'
    conf_fd = StringIO.StringIO(conf_str)
    config = ConfigParser.SafeConfigParser()
    config.readfp(conf_fd)
    options = {}
    for key_value in config.items("navigator"):
      options["navigator." + key_value[0]] = key_value[1]
    conf = MetadataConfig(options)
    assert_equal("http://nms_host:7187", conf.server_url)
    assert_false(conf.has_telemetry_publisher_url())

    conf_str = '[navigator]\n' + \
    'telemetry_publisher.server.url=http://tp_host:7187'
    conf_fd = StringIO.StringIO(conf_str)
    config = ConfigParser.SafeConfigParser()
    config.readfp(conf_fd)
    options = {}
    for key_value in config.items("navigator"):
      options["navigator." + key_value[0]] = key_value[1]
    conf = MetadataConfig(options)
    assert_equal("http://tp_host:7187", conf.server_url)
    assert_true(conf.has_telemetry_publisher_url)

    conf_str = '[navigator]\n' + \
    'navigator.client.serviceName=hive\n' + \
    'navigator.client.serviceType=HIVE\n' + \
    'navigator.client.roleName=hive-HIVESERVER2-aa5e025d506864e1aea297e46ffb16e3\n' + \
    'navigator.client.roleType=HIVESERVER2\n' + \
    'navigator.client.clusterName=cluster\n' + \
    'navigator.server.timeout_seconds=120'

    conf_fd = StringIO.StringIO(conf_str)
    config = ConfigParser.SafeConfigParser()
    config.readfp(conf_fd)
    options = {}
    for key_value in config.items("navigator"):
      options["navigator." + key_value[0]] = key_value[1]
    conf = MetadataConfig(options)
    assert_true(conf.server_url is None)
    assert_false(conf.has_telemetry_publisher_url())
