# Copyright (c) 2013-2014 Cloudera, Inc. All rights reserved.

import time
import os

from cmf.audit.navigator_plugin import AuditPlugin
from cmf.audit.navigator_plugin import MetadataPlugin
from cmf.audit.util import make_audit_plugin
from cmf.audit.util import make_metadata_plugin
from cmf.audit.spark.spark_lineage_parser import parse_spark_lineage_event
from nose.tools import assert_true, assert_false

def skiprhel5(original_function):
  """
  This decorator skips a test from being run if the underlying
  distro is RHEL5
  """
  def new_function():
      if is_rhel_5() is False:
        original_function()
  return new_function

def is_rhel_5():
  """
    This function, determines if the underlying distro is any version of
    Red-Hat 5. Unfortunately, the output of os.platform.platform() can be
    different on different platforms so an easier way is to check for the
    redhat release file on the machine, which provides a more deterministic
    way to identify RHEL5
  """
  path = '/etc/redhat-release'
  if(os.path.isfile(path) is False):
    return False
  # The contents of the file will be like:
  # Red Hat Enterprise Linux Server release 5.10 (Tikanga)
  rh_ver = open('/etc/redhat-release', 'r').read().split(' ')[6].split('.')
  return rh_ver[0] == '5'

def make_spark_metadata_plugin():
  return MetadataPlugin(parse_spark_lineage_event, [""], "spark_lineage_log_",
                        'spark', is_csd_process=True);

def wait_and_assert_true(condition, max_wait=30):
  """
  This function, takes in a condition that will be evaluated to true. The
  function spins till either the condition becomes true or the max-wait time
  is up.
  """
  waited = 0

  while waited <= max_wait and not condition():
    time.sleep(1)
    waited += 1

  assert_true(condition())

def wait_and_assert_false(condition, max_wait=30):
  """
  This function, takes in a condition that will be evaluated to false. The
  function spins till either the condition becomes false or the max-wait time
  is up.
  """

  wait_and_assert_true(lambda: not condition(), max_wait)

class TestAuditPlugin(object):

  def test_make_audit_plugin(self):
    """
    Test if the audit plugin is returned for specific and generic roles.
    """
    plugin1 = make_audit_plugin("impala-IMPALAD")
    assert_true(type(plugin1) is AuditPlugin)

    # Test that multiple processes will have separate plugins
    plugin2 = make_audit_plugin("impala-IMPALAD-subprocess")
    assert_true(type(plugin2) is AuditPlugin)

    assert_true(id(plugin1) != id(plugin2))

    plugin3 = make_audit_plugin("generic-PROCESS")
    assert_true(type(plugin3) is AuditPlugin)

    plugin4 = make_spark_metadata_plugin();

    assert_true(type(plugin4) is MetadataPlugin)
