#!/usr/bin/env python
# Copyright (c) 2011 Cloudera, Inc. All rights reserved.
# For testing, creates sample heartbeat response.

import avro.protocol
import avro.datafile
import os

MASTER_AGENT_PROTOCOL = avro.protocol.parse(
  file(os.path.join(os.path.dirname(__file__), "../cmf/master_agent.avpr")).read())
HEARTBEAT_RESPONSE_SCHEMA = MASTER_AGENT_PROTOCOL.types_dict.get("HeartbeatResponse")

r = {
  "hostname": "localhost",
  "processes": [
    {
      "id": 1,
      "user": "test_user",
      "group": "test_group",
      "name": "false",
      "program": "/usr/bin/false",
      "arguments": ["hello"],
      "status_links": { "status": "http://localhost:9876/status" },
      "one_off": True,
      # This is "hello.conf" -> "9876\n" in a zip (-0 compressed) file.
      "configuration_data": '',
    },
    {
      "id": 2,
      "user": "test_user",
      "group": "test_group",
      "name": "hello",
      "program": "hello/hello.py",
      "arguments": ["hello"],
      "status_links": { "status": "http://localhost:9876/status" },
      "one_off": False,
      # This is "hello.conf" -> "9876\n" in a zip (-0 compressed) file.
      "configuration_data": 'PK\x03\x04\n\x00\x00\x00\x00\x00\x16X\x16=\xdava\x17\x05\x00\x00\x00\x05\x00\x00\x00\n\x00\x15\x00hello.confUT\t\x00\x03\xcceqL\xcceqLUx\x04\x00\xf5\x01\x00\x009876\nPK\x01\x02\x17\x03\n\x00\x00\x00\x00\x00\x16X\x16=\xdava\x17\x05\x00\x00\x00\x05\x00\x00\x00\n\x00\r\x00\x00\x00\x00\x00\x00\x00\x00\x00\xb4\x81\x00\x00\x00\x00hello.confUT\x05\x00\x03\xcceqLUx\x00\x00PK\x05\x06\x00\x00\x00\x00\x01\x00\x01\x00E\x00\x00\x00B\x00\x00\x00\x00\x00',
    }
  ]
}

w = avro.datafile.DataFileWriter(
    file("response.avro", "w"),
    avro.io.DatumWriter(),
    HEARTBEAT_RESPONSE_SCHEMA)
w.append(r)
w.flush()
