# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
import cmf.config
from cmf.safety_valve import SafetyValve

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import os
import tempfile
import shutil
import simplejson

from cmf.monitor import conf, schema
from cmf.monitor.daemon import MonitorDaemon
import cmf.agent
from cmf.monitor.firehoses import Firehoses
from cmf.monitor.host.filesystem_map import FilesystemMap
from cmf.monitor.role_update import RoleUpdate
from cmf.heartbeat_tracker import HeartbeatTracker

class MockAgent:
  def __init__(self):
    self.hostname = "hostname"
    self.logfile = "/path/to/agent/log"
    self.parcel_dir_is_set = True

    self.supervisee_environment = dict(
      MGMT_HOME="/mgmt/home")
    self.supervisor_client = MockSupervisorWrapper()
    self.safety_valve = SafetyValve()
    self.cfg = cmf.config.AgentConfig()
    self.cfg.parcel_dir = "/path/to/agent/parcels"
    self.cfg.paths.process_dir = "/path/to/agent/process"
    self.cfg.metrics_url_timeout_seconds = 10.0
    self.cfg.task_metrics_timeout_seconds = 10.0
    self.cfg.stacks_log_bundle_max_bytes = 1024
    self.cfg.stacks_log_max_uncompressed_file_size_bytes = 1024
    self.heartbeat_tracker = HeartbeatTracker()

  def parcel_directory_in_use(self):
    return True

class MockSupervisorWrapper:
  def __init__(self):
    self.pid = None
    self.psutil_process = None
    self.stats = {"latency_sec": 0, "successes": 0, "failures": 0}

class MockFirehoses(Firehoses):

  def get_firehose_metrics(self, roletype):
    metrics = {}
    if roletype == "HOSTMONITOR":
      metrics[schema.HMON_MESSAGE_TRANSMIT_DURATION] = 10000
      metrics[schema.HMON_MESSAGE_TRANSMIT_SUCCEEDED] = 1000
      metrics[schema.HMON_MESSAGE_TRANSMIT_FAILED] = 100
      metrics[schema.HMON_MESSAGE_BYTES_SENT] = 10
    elif roletype == "SERVICEMONITOR":
      metrics[schema.SMON_MESSAGE_TRANSMIT_DURATION] = 20000
      metrics[schema.SMON_MESSAGE_TRANSMIT_SUCCEEDED] = 2000
      metrics[schema.SMON_MESSAGE_TRANSMIT_FAILED] = 200
      metrics[schema.SMON_MESSAGE_BYTES_SENT] = 20
    return metrics

def make_config_file(content, filename=conf.CLOUDERA_MONITOR_PROPERTIES):
  """
  Creates a file with the input name and contents in a tmp directory
  and returns the absolute path to that directory.
  """
  conf_dir = tempfile.mkdtemp(suffix='.scm-agent-test')
  try:
    path = os.path.join(conf_dir, filename)
    file(path, 'w').write(content)
    return conf_dir
  except:
    shutil.rmtree(conf_dir)
    raise

def replace_config_file(conf_dir, file_name, content):
  """
  Utility to replace content of specified tmp file. Creates it if it does
  not exist. Caller needs to do cleanup in case exception throws.
  """
  path = os.path.join(conf_dir, file_name)
  file(path, 'w').write(content)

class TestMonitorDaemon(MonitorDaemon):
  def __init__(self):
    fs_map = FilesystemMap(['aufs', 'overlay'])
    MonitorDaemon.__init__(self,  "host", "host", fs_map, agent=MockAgent())

def parse_metrics_sample(adapter, conf, version):
  sample = adapter.get_metrics_sample(version)
  return parse_metrics_for_provided_sample(adapter, conf, version, sample)

def get_role_update_for_sample(adapter, conf, version):
  sample = adapter.get_metrics_sample(version)
  return get_role_update_for_provided_sample(adapter, conf, version, sample)

def parse_metrics_for_provided_sample(adapter, conf, version, sample):
  update = get_role_update_for_provided_sample(adapter, conf, version, sample)
  metrics = { }
  for metric in update.to_avro_dict()['metrics']:
    metrics[metric['id']] = metric['value']
  return metrics

def get_role_update_for_provided_sample(adapter, conf, version, sample):
  metrics = adapter.parse_metrics_from_url(
    conf, simplejson.load(StringIO(sample)))
  update = RoleUpdate(name="whatever_role-1",
                      roletype=adapter.role_type,
                      servicetype=adapter.service_type,
                      service_name="whatever_service-1")
  adapter.add_webserver_metrics(version,
                                update,
                                metrics,
                                accessors=TestMonitorDaemon().metric_accessors)
  return update
