# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

import os
import subprocess
import time

from cmf.monitor import proc_metrics_utils as utils
from cmf.monitor import schema
from nose.tools import assert_true, assert_equals, assert_false

def test_merge_cpu_results():
  """ Test the we handle processes coming and going correctly"""
  # First test with no previous values
  metrics = { 40 : { schema.TOTAL_CPU_USER : 5,
                     schema.TOTAL_CPU_USER_WITH_DESCENDANTS : 10,
                     schema.TOTAL_CPU_SYSTEM : 15,
                     schema.TOTAL_CPU_SYSTEM_WITH_DESCENDANTS : 20 },
              60 : { schema.TOTAL_CPU_USER : 5,
                     schema.TOTAL_CPU_USER_WITH_DESCENDANTS : 10,
                     schema.TOTAL_CPU_SYSTEM : 15,
                     schema.TOTAL_CPU_SYSTEM_WITH_DESCENDANTS : 20 },
            }
  cpu_metrics = utils._get_with_descendants_cpu_metrics(40, metrics, {}, None, None)
  assert_equals(2, len(cpu_metrics))
  assert_equals(30, cpu_metrics[0])
  assert_equals(70, cpu_metrics[1])


  # Test with a race condition causing it to lose data so that it has to use
  # the floor value.
  # In this case there's no race, with desecendants has the pid 60s values.
  # We add a little to indicate that pid 60 did work in the past second. (1
  # cpu second for both user and system
  new_metrics = { 40 : { schema.TOTAL_CPU_USER : 20,
                         schema.TOTAL_CPU_USER_WITH_DESCENDANTS : 26,
                         schema.TOTAL_CPU_SYSTEM : 35,
                         schema.TOTAL_CPU_SYSTEM_WITH_DESCENDANTS : 56 },
                  80 : { schema.TOTAL_CPU_USER : 12,
                         schema.TOTAL_CPU_USER_WITH_DESCENDANTS : 22,
                         schema.TOTAL_CPU_SYSTEM : 32,
                         schema.TOTAL_CPU_SYSTEM_WITH_DESCENDANTS : 42 },
                 }
  cpu_metrics = utils._get_with_descendants_cpu_metrics(40,
                                                        new_metrics,
                                                        metrics,
                                                        30,
                                                        70)
  assert_equals(2, len(cpu_metrics))
  # Should be the original value plus the new CPU from pid 40 and the new from 80
  assert_equals(30 + 15 + 1 + 34, cpu_metrics[0])
  assert_equals(70 + 20 + 1 + 74, cpu_metrics[1])

  # Test that pid 40 didn't get 60s values (ie, the race condition)
  # That means we lose the extra 1 CPU second for user and system
  new_metrics = { 40 : { schema.TOTAL_CPU_USER : 20,
                         schema.TOTAL_CPU_USER_WITH_DESCENDANTS : 10,
                         schema.TOTAL_CPU_SYSTEM : 35,
                         schema.TOTAL_CPU_SYSTEM_WITH_DESCENDANTS : 20 },
                  80 : { schema.TOTAL_CPU_USER : 12,
                         schema.TOTAL_CPU_USER_WITH_DESCENDANTS : 22,
                         schema.TOTAL_CPU_SYSTEM : 32,
                         schema.TOTAL_CPU_SYSTEM_WITH_DESCENDANTS : 42 },
                 }
  cpu_metrics = utils._get_with_descendants_cpu_metrics(40,
                                                        new_metrics,
                                                        metrics,
                                                        30,
                                                        70)
  assert_equals(2, len(cpu_metrics))
  # Should be the original value plus the new CPU from pid 40 and the new from 80.
  assert_equals(30 + 15 + 34, cpu_metrics[0])
  assert_equals(70 + 20 + 74, cpu_metrics[1])

def test_host_fd_stats():
  fd_open, fd_max = utils.get_host_fd_stats()
  assert_false(fd_open is None)
  assert_false(fd_max is None)
