#!/usr/bin/env python
# Copyright (c) 2016 Cloudera, Inc. All rights reserved.

from nose.tools import assert_true, assert_false
from cmf.monitor.generic.utils import VersionUtils, VersionType

def test_VersionUtils():
    """
    Tests the version comparison utility.
    """
    assert_true(VersionUtils.at_least("5.10", "5.4", VersionType.MINOR))
    assert_false(VersionUtils.at_least("5.2", "5.4", VersionType.MINOR))
    assert_true(VersionUtils.at_least("5.10.0", "5.9.0", VersionType.MINOR))

    assert_true(VersionUtils.at_least("6.0", "5.4", VersionType.MAJOR))
    assert_true(VersionUtils.at_least("5.10.1", "5.10.0", VersionType.PATCH))

    assert_true(VersionUtils.less_than("5.4", "5.10", VersionType.MINOR))
    assert_false(VersionUtils.less_than("5.9.1", "5.4", VersionType.MINOR))
    assert_false(VersionUtils.less_than("5.10.0", "5.9.1", VersionType.MINOR))

    assert_true(VersionUtils.less_than("5.9.0", "6.0.0", VersionType.MAJOR))
    assert_true(VersionUtils.less_than("5.10.0", "5.10.1", VersionType.PATCH))

    # Test the truncation logic for different version types
    assert_true(VersionUtils.at_least("6.0.2,", "6.10.1", VersionType.MAJOR))
    assert_false(VersionUtils.at_least("6.0.2", "6.10.1", VersionType.MINOR))
    assert_false(VersionUtils.at_least("6.0.2", "6.10.1", VersionType.PATCH))

    assert_false(VersionUtils.less_than("6.0.2", "6.10.1", VersionType.MAJOR))
    assert_true(VersionUtils.less_than("6.0.2", "6.10.1", VersionType.MINOR))
    assert_true(VersionUtils.less_than("6.0.2", "6.10.1", VersionType.PATCH))

    # Test versions with 'x'
    assert_false(VersionUtils.less_than("6.0.x", "6.0.0", VersionType.PATCH))
    assert_true(VersionUtils.at_least("6.x.0", "6.10.0", VersionType.MINOR))
