# Copyright (c) 2016 Cloudera, Inc. All rights reserved.
import os
import shutil
import time

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.monitor.abstract_monitor import CollectionError
from cmf.monitor import constants
from cmf.monitor.generic.adapter import Adapter
from cmf.monitor.generic.metric_collectors import WebServerMetricCollector
from cmf.monitor.monitor_properties import ConfParser
from cmf.monitor.conf import CLOUDERA_MONITOR_PROPERTIES
from cmf.monitor.role_update import RoleUpdate
from cmf.safety_valve import SafetyValve
from cmf_test.monitor import make_config_file


from mock import Mock

from nose.tools import assert_true, assert_equals, assert_false

class MockAdapter(Adapter):
  def __init__(self):
    Adapter.__init__(self, "SOMESERVICE", "SOMEROLE", SafetyValve())
    self._metrics_url = None

  def get_metrics_url(self, conf):
    return self._metrics_url

  def set_metrics_url(self, metrics_url):
    self._metrics_url = metrics_url

  def parse_metrics_from_url(self, conf, json):
    extracted_metrics = dict()
    for metric in json.get('metrics', []):
      metricid = metric.get('id', None)
      if metricid is None:
        continue
      value = metric.get('value', None)
      if value is None:
        continue
      extracted_metrics[metricid] = float(value)
    return extracted_metrics

  def add_webserver_metrics(self,
                              service_version,
                              update,
                              collected_metrics,
                              metric_accessors):
    for metricid, value in collected_metrics.iteritems():
      update.add_metric(metricid, value)

class MockCollector(WebServerMetricCollector):

  def __init__(self, adapter, username_attr, password_attr, open_url_call_out):
    WebServerMetricCollector.__init__(self, "somerole", adapter, "metric_accessors",
                                      username_attr, password_attr)
    self._error = None
    self._url_response = None
    self._urlopen_callout = open_url_call_out

  def set_url_response(self, url_response):
    self._url_response = url_response

  def _open_url(self):
    WebServerMetricCollector._open_url(self)
    return StringIO(self._url_response)

def test_url_open():
  conf ="""
[somerole]
role_name = someservice-somerole-test
service_name = someservice
host = localhost
service_version = 4
collect_interval = 1.0
log_dir = /path/to/logs
username = %s
password = %s
"""

  # Make two different configs
  conf_dir = make_config_file(conf % ("foo", "bar"))
  conf_dir2 = make_config_file(conf % ("foo", "boo"))

  url_response = """
{
"metrics": [
  { "id": 66666, "value": -3.14 },
  { "id": 66667, "value": 3.15 }
]
}
"""

  try:
    url_open = Mock()
    adapter = MockAdapter()
    collector = MockCollector(adapter, "username", "password", url_open)
    collector.set_url_response(url_response)
    url = "http://foo.com/metrics"
    adapter.set_metrics_url(url)

    conf_parser = ConfParser(conf_dir, CLOUDERA_MONITOR_PROPERTIES)
    collector.update_with_conf(conf_parser)
    collector.collect_and_parse(conf_parser)
    assert_true(url_open.called)
    assert_equals(url, url_open.call_args[1]["url"])
    assert_equals("foo", url_open.call_args[1]["username"])
    assert_equals("bar", url_open.call_args[1]["password"])

    url_open.reset_mock()
    conf_parser = ConfParser(conf_dir2, CLOUDERA_MONITOR_PROPERTIES)
    collector.update_with_conf(conf_parser)
    collector.collect_and_parse(conf_parser)
    assert_true(url_open.called)
    assert_equals(url, url_open.call_args[1]["url"])
    assert_equals("foo", url_open.call_args[1]["username"])
    assert_equals("boo", url_open.call_args[1]["password"])
  finally:
    shutil.rmtree(conf_dir)
    shutil.rmtree(conf_dir2)
