#!/usr/bin/env python
# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

import diewithparent
import logging
import multiprocessing
import os
import signal
import subprocess
import time

LOG = logging.getLogger(__name__)

def inner_child(queue, set_diewithparent):
  if set_diewithparent:
    pre_fn = diewithparent.diewithparent
  else:
    pre_fn = None

  p = subprocess.Popen(['/bin/sleep', '1000'], preexec_fn=pre_fn)
  queue.put(p.pid)
  p.wait()

def run_children(set_diewithparent):
  q = multiprocessing.Queue()
  p = multiprocessing.Process(target=inner_child, args=(q, set_diewithparent))
  p.start()
  c_pid = p.pid
  gc_pid = q.get()

  LOG.info("c pid is %s, gc pid is %s" % (c_pid, gc_pid))

  os.kill(c_pid, signal.SIGKILL)

  p.join()

  return gc_pid

def ensure_no_grandchild(gc_pid):
  # Need to sleep to ensure Grandchild is really gone. We can't
  # synchronize with wait() as it's no longer in our subtree.
  time.sleep(0.5)

  try:
    # Now we expect it to be gone and kill to fail.
    os.kill(gc_pid, 0)
  except OSError:
    return

  assert False, "Not reached"

def test_diewithparent():
  """Test that diewithparent actually works"""

  gc_pid = run_children(True)

  ensure_no_grandchild(gc_pid)

def test_no_diewithparent():
  """Test that without diewithparent, the child doesn't die"""

  gc_pid = run_children(False)

  os.kill(gc_pid, signal.SIGTERM)

  ensure_no_grandchild(gc_pid)

if __name__ == '__main__':
  test_no_diewithparent()
  test_diewithparent()
