#!/usr/bin/env python
# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

from nose.tools import assert_true, assert_equal, assert_false, assert_raises

from thrift.protocol import TCompactProtocol
from thrift.transport import TTransport
from thrift.TSerialization import deserialize
import base64
import sys
import zlib

# This tests whether we can serialize, deserialize, compress, and decompress
# Impala's thrift runtime profile in Python
def test_impala_thrift():

  # This is less than ideal, but hopefully we'll figure out what we want
  # to do with the generate python and the PYTHONPATH when we write the
  # non-test code.
  sys.path.append('src/cmf/gen-py')
  from RuntimeProfile.ttypes import TRuntimeProfileTree, TRuntimeProfileNode 

  # Create the profile
  node = TRuntimeProfileNode()
  node.name = "QueryId"
  # Add a couple nodes to the tree so we have enough data to actually
  # do some compression
  tree = TRuntimeProfileTree([node, node, node, node, node])

  # Serialize
  transportOut = TTransport.TMemoryBuffer()
  protocolOut = TCompactProtocol.TCompactProtocol(transportOut)
  tree.write(protocolOut)
  bytes = transportOut.getvalue()

  # Compress
  compressed = zlib.compress(bytes)

  # Decompress
  decompressed = zlib.decompress(compressed)

  # Make sure that the compression and decompression process worked right
  assert_equal(bytes, decompressed)
  assert_true(len(compressed) < len(decompressed))

  # Deserialize
  transportIn = TTransport.TMemoryBuffer(decompressed)
  protocolIn = TCompactProtocol.TCompactProtocol(transportIn)
 
  newTree = TRuntimeProfileTree()
  newTree.read(protocolIn)
  
  # Make sure we got the data out right 
  assert_equal("QueryId", newTree.nodes[0].name)

def test_impala_log_format():
  """ Test the decoding the format we will get from the impala log"""

  sys.path.append('src/cmf/gen-py')
  from RuntimeProfile.ttypes import TRuntimeProfileTree, TRuntimeProfileNode

  try:
    fd = open('src/cmf_test/impala-test-data/encoded-runtime-profiles/DEFAULT_QUERY')
    tree = TRuntimeProfileTree()
    deserialize(tree,
                zlib.decompress(base64.b64decode(fd.readline())),
                protocol_factory=TCompactProtocol.TCompactProtocolFactory())

    assert_true(len(tree.nodes) > 0)
  finally:
    fd.close()
