# Copyright (c) 2014 Cloudera, Inc. All rights reserved.

import filecmp
import os
import random
import shutil
import string
import tempfile

from nose.tools import assert_equals, assert_true

from cmf.log_compressor import get_compressed_logs
from cmf.zip_util import ZipFile
import cmf.redaction

def _write_file(path, size):
  data = ''.join(random.choice(string.ascii_uppercase) for i in range(1024))
  f = open(path, "w")
  try:
    total = 0
    while total < size:
      f.write(data)
      total += len(data)
  finally:
    if f:
      f.close()

def _extract_file(zipped, path, output_path):
  f = None
  try:
    f = open(output_path, "w")
    f.write(zipped.read(path))
  finally:
    if f:
      f.close()

def test_get_compressed_logs():
  zipped = None
  tmpdir = tempfile.mkdtemp()
  try:
    f1 = os.path.join(tmpdir, 'f1')
    f2 = os.path.join(tmpdir, 'f2')
    f3 = os.path.join(tmpdir, 'f3')
    _write_file(f1, 10 * 1024)
    _write_file(f2, 10 * 1024)
    _write_file(f3, 10 * 1024)
    bundle = get_compressed_logs([f1, f2, f3],
                                 25 * 1024,
                                 tmpdir,
                                 cmf.redaction.NOP_REDACTOR)
    zipped = ZipFile(bundle)
    assert_equals(10 * 1024, zipped.getinfo(f1).file_size)
    assert_equals(10 * 1024, zipped.getinfo(f2).file_size)
    assert_equals(5 * 1024, zipped.getinfo(f3).file_size)
    for f in [f1, f2]:
      extracted = "%s-extracted" % f
      _extract_file(zipped, f, extracted)
      assert_true(filecmp.cmp(f, extracted))
  finally:
    if zipped is not None:
      zipped.close()
    shutil.rmtree(tmpdir)
