# Copyright (c) 2015 Cloudera, Inc. All rights reserved.

import simplejson as json

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.metric_registry import Counter, SimpleGauge, SamplingGauge, \
  METRIC_REGISTRY

from nose.tools import assert_true, assert_equals, assert_false

def test_counter():
  counter = Counter("counter", "A counter", "tests")
  METRIC_REGISTRY.add_counter("test_counter", counter)
  assert_equals(0, counter.get_count())
  counter.increment(7)
  counter.decrement(2)
  counter = METRIC_REGISTRY.get_counter("test_counter")
  assert_true(isinstance(counter, Counter))
  assert_equals(5, counter.get_count())
  counter = Counter("counter",
                    "A counter with initial value",
                    "tests",
                    initial_value=4)
  assert_equals(4, counter.get_count())
  counter2 = Counter("counter", "A counter", "tests")
  assert_equals(0, counter2.get_count())
  counter = METRIC_REGISTRY.get_or_add_counter("test_counter", counter2)
  # we should get back the original one.
  assert_equals(5, counter.get_count())
  METRIC_REGISTRY.reset()

def test_simple_gauge():
  gauge = SimpleGauge("simple_gague", "simple_gauge", "tests")
  METRIC_REGISTRY.add_gauge("test_simple_gauge", gauge)
  assert_equals(0, gauge.get_value())
  gauge.set_value(5)
  gauge = METRIC_REGISTRY.get_gauge("test_simple_gauge")
  assert_true(isinstance(gauge, SimpleGauge))
  assert_equals(5, gauge.get_value())
  gauge = SimpleGauge("simple_gague", "simple_gauge", "tests", initial_value=4)
  assert_equals(4, gauge.get_value())

  gauge2 = SimpleGauge("simple_gague", "simple_gauge", "tests")
  assert_equals(0, gauge2.get_value())
  gauge = METRIC_REGISTRY.get_or_add_gauge("test_simple_gauge", gauge2)
  # we should get back the original one.
  assert_equals(5, gauge.get_value())
  METRIC_REGISTRY.reset()

def test_sampling_gauge():
  call_count = 0
  gauge = SamplingGauge("sampling_gague",
                        "A sampling gauge",
                        "samples",
                        lambda: call_count)
  METRIC_REGISTRY.add_gauge("test_sampling_gauge", gauge)
  assert_equals(0, gauge.get_value())
  call_count += 1
  assert_equals(1, gauge.get_value())
  gauge = METRIC_REGISTRY.get_gauge("test_sampling_gauge")
  assert_true(isinstance(gauge, SamplingGauge))
  assert_equals(1, gauge.get_value())
  saw_exception = None
  try:
    SamplingGauge("sampling_gague", "A sampling gauge", "samples", callback=7)
    # We should not get here.
    saw_exception = False
  except Exception, e:
    saw_exception = True
  assert_true(saw_exception)
  METRIC_REGISTRY.reset()

def test_dump_metrics():
  counter1 = Counter("counter_1", "First counter", "tests", initial_value=41)
  counter2 = Counter("counter_2", "Second counter", "units", initial_value=43)
  METRIC_REGISTRY.add_counter("counter1", counter1)
  METRIC_REGISTRY.add_counter("counter2", counter2)
  simple_gauge = SimpleGauge("simple_gague",
                             "A simple gauge",
                             "tests",
                             denominator="second",
                             initial_value=17)
  sampling_gauge = SamplingGauge("sampling_gague",
                                 "A sampling gauge",
                                 "samples",
                                 lambda: 42)
  METRIC_REGISTRY.add_gauge("simple_gauge", simple_gauge)
  METRIC_REGISTRY.add_gauge("sampling_gauge", sampling_gauge)
  metrics_json = METRIC_REGISTRY.dump_metrics()
  metrics_dict = json.load(StringIO(metrics_json))
  assert_true("counters" in metrics_dict)
  assert_true("gauges" in metrics_dict)
  counters = metrics_dict["counters"]
  gauges = metrics_dict["gauges"]
  [assert_true(key in counters) for key in ("counter1", "counter2")]
  [assert_true(key in gauges) for key in ("simple_gauge", "sampling_gauge")]
  assert_equals(43, counters["counter2"]["count"])
  assert_equals("counter_1", counters["counter1"]["label"])
  assert_equals("Second counter", counters["counter2"]["description"])
  assert_equals("tests", counters["counter1"]["numerator"])
  assert_equals(None, counters["counter1"]["denominator"])
  assert_equals(17, gauges["simple_gauge"]["value"])
  assert_equals("tests", gauges["simple_gauge"]["numerator"])
  assert_equals("second", gauges["simple_gauge"]["denominator"])
  assert_equals(42, gauges["sampling_gauge"]["value"])
  assert_equals("samples", gauges["sampling_gauge"]["numerator"])
