#!/usr/bin/env python
# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

from nose.tools import assert_true, assert_equal, assert_false, assert_raises, raises

from cmf.parcel_id import ParcelId

def test_typical():
  url = ParcelId.url("https://archive.cloudera.com/cdh4/rhel6/foo-1.0.0-rhel6.parcel")
  assert_equal("foo-1.0.0-rhel6.parcel", url.get_filename())
  assert_equal("foo", url.product)
  assert_equal("1.0.0", url.version)
  assert_equal("rhel6", url.distro)
  assert_false(url.is_diff)
  assert_true(url.get_base_parcel_filename() is None)

def test_embedded_dashes():
  url = ParcelId.url("https://archive.cloudera.com/cdh4/rhel6/foo-1.0.0-cdh1.0.0.p0.100-rhel6.parcel")
  assert_equal("foo-1.0.0-cdh1.0.0.p0.100-rhel6.parcel", url.get_filename())
  assert_equal("foo", url.product)
  assert_equal("1.0.0-cdh1.0.0.p0.100", url.version)
  assert_equal("rhel6", url.distro)
  assert_false(url.is_diff)
  assert_true(url.get_base_parcel_filename() is None)

def test_no_path():
  url = ParcelId.url("https://archive.cloudera.com/foo-1.0.0-rhel6.parcel")
  assert_equal("foo-1.0.0-rhel6.parcel", url.get_filename())
  assert_equal("foo", url.product)
  assert_equal("1.0.0", url.version)
  assert_equal("rhel6", url.distro)
  assert_false(url.is_diff)
  assert_true(url.get_base_parcel_filename() is None)

@raises(Exception)
def test_bad_ext():
  url = ParcelId.url("https://archive.cloudera.com/boo/foo-1.0.0-deb.bleh")

@raises(Exception)
def test_no_distro():
  url = ParcelId.url("https://archive.cloudera.com/boo/foo-1.0.0.parcel")

@raises(Exception)
def test_no_version():
  url = ParcelId.url("https://archive.cloudera.com/boo/foo-rhel6.parcel")

@raises(Exception)
def test_no_ext():
  url = ParcelId.url("https://archive.cloudera.com/cdh4/rhel6/foo-1.0.0-rhel6")

def test_diff_typical():
  url = ParcelId.url("https://archive.cloudera.com/cdh4/rhel6/foo-1.0.0-1.1.0-rhel6.diff")
  assert_equal("foo-1.0.0-1.1.0-rhel6.diff", url.get_filename())
  assert_equal("foo", url.product)
  assert_equal("1.1.0", url.version)
  assert_equal("1.0.0", url.base_version)
  assert_equal("rhel6", url.distro)
  assert_true(url.is_diff)
  assert_equal("foo-1.0.0-rhel6.parcel", url.get_base_parcel_filename())

def test_diff_with_embedded_dashes():
  url = ParcelId.url("https://archive.cloudera.com/cdh4/rhel6/CDH-4.3.0-1.cdh4.3.0.p0.181-4.3.0-1.cdh4.3.0.p1.181-el6.diff")
  assert_true(url.is_diff)

  assert_equal("CDH-4.3.0-1.cdh4.3.0.p0.181-4.3.0-1.cdh4.3.0.p1.181-el6.diff", url.get_filename())
  assert_equal("CDH", url.product)
  assert_equal("4.3.0-1.cdh4.3.0.p1.181", url.version)
  assert_equal("4.3.0-1.cdh4.3.0.p0.181", url.base_version)
  assert_equal("el6", url.distro)
  assert_equal("CDH-4.3.0-1.cdh4.3.0.p0.181-el6.parcel", url.get_base_parcel_filename())

@raises(Exception)
def test_diff_with_embedded_dashes_mismatched_dashes():
  url = ParcelId.url("https://archive.cloudera.com/cdh4/rhel6/CDH-4.3.0-serializationishard-1.cdh4.3.0.p0.181-4.3.0-1.cdh4.3.0.p1.181-el6.diff")

def test_dir():
  pid = ParcelId.dir("foo-1.0.0-cdh1.0.0.p0.100")
  assert_equal("foo", pid.product)
  assert_equal("1.0.0-cdh1.0.0.p0.100", pid.version)
  assert_equal(None, pid.distro)
  assert_false(pid.is_diff)
