# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
from cmf.safety_valve import SafetyValve

from nose.tools import assert_equals, assert_raises

def test_update_safety_valve():

  safety_valve = SafetyValve()

  assert_raises(Exception, safety_valve.update_safety_valve, None)

  safety_valve.update_safety_valve({})
  current_dict = safety_valve.get_safety_valve_dictionary()
  assert_equals(0, len(current_dict))
  assert_equals(None, safety_valve.get("key_does_not_exist"))
  assert_equals(42, safety_valve.get("key_does_not_exist", 42))

  safety_valve_str = \
"""
illegal_line

 =

use_generic_monitor_for_noroletype
use_generic_monitor_for_srv_role    =
use_generic_monitor_for_service_role    = foobar = value
"""
  safety_valve.update_safety_valve(
    dict(host_agent_safety_valve=safety_valve_str))
  current_dict = safety_valve.get_safety_valve_dictionary()
  assert_equals(2, len(current_dict))
  assert_equals('', current_dict.get('use_generic_monitor_for_srv_role'))
  assert_equals(' foobar = value',
                current_dict.get('use_generic_monitor_for_service_role'))
  assert_equals('', safety_valve.get('use_generic_monitor_for_srv_role'))
  assert_equals(' foobar = value',
                safety_valve.get('use_generic_monitor_for_service_role'))
  safety_valve_str = \
"""
illegal_line

 =

use_generic_monitor_for_noroletype
use_generic_monitor_for_srv_role    =
use_generic_monitor_for_service_role    = foobar = value
use_generic_monitor_for_srv_role = true
use_generic_MONITOR_FOR_srv_role = tRuE
BARFOO=FOOBAR
"""
  safety_valve.update_safety_valve(
    dict(host_agent_safety_valve=safety_valve_str))
  current_dict = safety_valve.get_safety_valve_dictionary()
  # keys are case sensitive.
  assert_equals(4, len(current_dict))
  assert_equals(' true', current_dict.get('use_generic_monitor_for_srv_role'))
  assert_equals(' tRuE', current_dict.get('use_generic_MONITOR_FOR_srv_role'))
  assert_equals('FOOBAR', current_dict.get('BARFOO'))
  assert_equals(' true', safety_valve.get('use_generic_monitor_for_srv_role'))
  assert_equals(' tRuE', safety_valve.get('use_generic_MONITOR_FOR_srv_role'))
  assert_equals('FOOBAR', safety_valve.get('BARFOO'))

  safety_valve.update_safety_valve(
    dict(host_agent_safety_valve=""))
  current_dict = safety_valve.get_safety_valve_dictionary()
  assert_equals(0, len(current_dict))
