# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import

from flood.util.error import Error

TORRENT_SCHEMA = None  # Initialized below

import avro.ipc

class TorrentError(Error, avro.ipc.AvroRemoteException):
  def __init__(self, *pargs, **kwargs):
    Error.__init__(self, *pargs, **kwargs)
    avro.ipc.AvroRemoteException.__init__(self, str(self))

  @property
  def description(self):
    return str(self)

def __init_fn__():
  '''
  libtorrent as a python package is itself just a python binding,
  but it depends on libtorrent-rasterbar.so - which is built and installed
  inside "lib" that hosts the virtual environment that flood is in.
  This library won't be found by normal search mechanisms, and so
  it is explicitly pre-loaded here.  The location of this library will
  be found in the "lib" directory in sys.prefix

  All of this code (including imports) is inside a function so as to not
  pollute the namespace of this package.
  '''
  LIBTORRENT = "libtorrent-rasterbar.so"
  import ctypes
  import sys
  import os.path
  p = os.path.join(sys.prefix, 'lib', LIBTORRENT)
  try:
    if os.path.exists(p):
      ctypes.cdll.LoadLibrary(p)
  except:
    pass

  import os.path
  import avro.protocol
  global TORRENT_SCHEMA
  avprPath = os.path.join(os.path.dirname(__file__), "torrent.avpr")
  TORRENT_SCHEMA = avro.protocol.parse(file(avprPath).read())


__init_fn__()