# Copyright (c) 2015 Cloudera, Inc. All rights reserved.

from __future__ import absolute_import

import threading


class AsyncExec(object):
  '''
  Asynchronously execute some action, once per each object in "targets".
  The action to be executed is defined by the "run" method (of a sub-class
  of this class).

  Once all actions have been executed a "complete" method is invoked.
  '''

  class Instance(object):
    def __init__(self, idx, target):
      self.idx = idx
      self.target = target
      self.result = None
      self.exc = None

  def execute(self, targets):
    jobs = []
    for (idx, t) in enumerate(targets):
      inst = self.Instance(idx, t)
      thr = threading.Thread(target=self.__run, args=(inst,))
      thr.setDaemon(True)
      inst.thr = thr
      jobs.append(inst)
      thr.start()

    for j in jobs:
      j.thr.join()


    for j in jobs:
      self.complete(j.idx, j.target, j.result, j.exc)

  def __run(self, inst):
    '''
    Internal wrapper for "run()".
    Captures return value/exceptions to be passed to "complete".
    :param inst:
    :return:
    '''
    try:
      inst.result = self.run(inst.idx, inst.target)
    except Exception, e:
      inst.exc = e

  def run(self, idx, target):
    '''
    Perform an action for the provided "target" object.
    :param idx:   Index of "target" within "targets" passed to "execute".
    :param target: One of the cells of "targets as passed to "execute".
    :return: Any - user defined in override.
    '''
    return

  def complete(self, idx, target, result, exc):
    '''
    Processes the result of a "run" invocation on the calling thread
    within "execute".
    :param idx:    Index of target object as per "run".
    :param target: Target object as per "run"
    :param result: Return value from "run", if any.
    :param exc:    Exception from "run", if any.
    :return: Any - user defined in override.
    '''
    return
