# Copyright (c) 2015 Cloudera, Inc. All rights reserved.
from __future__ import absolute_import, print_function


class lazy(object):
  '''
  Decorator for lazy/deferred attribute calculation.
  '''
  def __init__(self, fn):
    self.fn = fn

  def __get__(self, instance, owner):
    val = self.fn(instance)
    setattr(instance, self.fn.__name__, val)
    return val



def main():

  class TestClass(object):
    execCount = 0
    @lazy
    def propA(self):
      self.execCount += 1
      return 2

  x = TestClass()
  assert x.propA == 2
  assert x.execCount == 1
  assert x.propA == 2
  assert x.execCount == 1


if __name__ == "__main__":
  main()