#!/bin/bash
# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

# This script is passed as a value to the -XX:OnOutOfMemoryError JVM option.
# The JVM could be a standalone java process or a JVM embedded in a C/C++
# process such as Impala catalogd and impalad.
# Upon OutOfMemoryError, the JVM will fork a new process and exec this script.
# This script sends SIGKILL to the JVM process, which is normally the parent
# process of the script process, but can be the "grandparent" process if the
# system shell /bin/sh points to the dash shell. The script also creates a
# file in the process' current working directory indicating that it exited
# due to an OOM.
# Note: We use this script instead of directly passing the "kill -9 %p" value
# because several CDH startup scripts do not double-quote the command line
# argument containing this value. This leads to a broken JVM command line.

# Enable tracing
set -x

# The command of this script
mycmd=$(ps -p $$ -o comm=)

# if dash is not the system shell, the target to kill is the parent process
TARGET=$PPID

# If dash is the system shell, there is an extra process between us and the
# standalone JVM process or the process with the embedded JVM (such as
# catalogd and impalad) we want to kill.
PCMD=$(ps -p $PPID -o comm=)
PARGS=$(ps -p $PPID -o args=)
if [ "$PCMD" == "sh" ] && [[ "$PARGS" == *"$mycmd"* ]]; then
  # If the command of the parent process is "sh" and its args contain the name
  # of this script, then it's dash. Find its parent's pid.
  TARGET=$(ps -p $PPID -o ppid=)

  # skip if it's supervisord
  if ps -p $TARGET -o args= | grep supervisord; then
    echo $(date) $PPID $TARGET > not_killed_by_killparent_on_oom
    exit 1
  fi
fi

echo $(date) > killed_by_killparent_on_oom
kill -9 $TARGET
