#!/bin/bash

# Copyright (c) 2015 Cloudera, Inc. All rights reserved.

# validate_writable_empty_dirs true|false dir...
#
# Checks for the existence of a specified list of local directories.
# If a given directory exists, verifies that it is writable by the
# current user and group. If the directory is non-empty, clears it
# if $1 is "true", returns an error otherwise.

# Time marker for both stderr and stdout
date; date 1>&2

# Enable tracing.
set -x

# Error exit codes
EXIT_GENERAL_ERROR=1
EXIT_NOT_A_DIR=2
EXIT_DIR_NOT_WRITABLE=3
EXIT_DIR_NOT_EMPTY=4

if [ $# -lt 2 ]; then
  echo "No directories to validate specified."
  exit $EXIT_GENERAL_ERROR
fi

for VALIDATE_DIR in "${@:2}"; do
  if [ -e $VALIDATE_DIR ]; then
    if [ ! -d $VALIDATE_DIR ]; then
      echo "Validation failed: $VALIDATE_DIR exists, but is not a directory."
      exit $EXIT_NOT_A_DIR
    fi
    if [ ! -w $VALIDATE_DIR ]; then
      echo "Validation failed: directory $VALIDATE_DIR exists, but is not writable by `whoami`."
      exit $EXIT_DIR_NOT_WRITABLE
    fi

    non_empty_dir=`find -L "$VALIDATE_DIR" -maxdepth 0 -not -empty`
    if [ -n "$non_empty_dir" ]; then
      if [ "$1" = "true" ]; then
        echo "$VALIDATE_DIR is not empty, clearing it."
        rm -rf $VALIDATE_DIR/*
      else
        echo "Validation failed: directory $VALIDATE_DIR exists and is writable by `whoami`, but is not empty."
        exit $EXIT_DIR_NOT_EMPTY
      fi
    fi
  fi
done
echo "Validation of ${@:2} succeeded: all existing directories are writable, and are empty or have been cleared."
exit 0
