#!/bin/bash

# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

get_gc_args() {
  # CSDs can supply JVM GC args here [ applicable for CDH >=6.3.0 ]

  # Uncomment below to override GC arguments provided by cloudera-config.sh:
  ##  JAVA8_GC_TUNING_ARGS="GC argument for JAVA8"
  ##  JAVA11_GC_TUNING_ARGS="GC argument for JAVA8"

  # Setting BASIC_GC_TUNING_ARGS to NULL for Java 8
  # As few CSDs, eg. Kakfa already have GC args fixed for Java 8 under:
  # /opt/cloudera/parcels/CDH-6.1.0-1.cdh6.1.0.p0.770702/lib/kafka/bin/kafka-run-class.sh
  # that collide with GC parameters for Java 8 (like 'UseParNewGC').
  # Hence, leaving it empty until CSDs decide to pick up GC args from here.
  JAVA8_GC_TUNING_ARGS=" "

  # function from cloudera-config.sh provides BASIC_GC_TUNING_ARGS based on java version
  set_basic_gc_tuning_args_based_on_java_version
}

# attempt to find java
locate_java_home

# For JVM-based processes, search-replace {{PID}} in heap dump path,
if [ -n "$CSD_JAVA_OPTS" ]; then
  # adding GC args
  get_gc_args
  CSD_GC_ARGS="$BASIC_GC_TUNING_ARGS"
  CSD_JAVA_OPTS+=" $CSD_GC_ARGS"

  export CSD_JAVA_OPTS=$(replace_pid $CSD_JAVA_OPTS)
  echo "Using $CSD_JAVA_OPTS as CSD_JAVA_OPTS"
fi

# Load the parcel environment
source_parcel_environment

echo "Using $CONF_DIR as conf dir"
echo "Using $CM_CSD_SCRIPT as process script"

# Search-replace {{CMF_CONF_DIR}} in files
replace_conf_dir

# mark common scripts as executable
make_scripts_executable

RUN_DIR=$CONF_DIR
if [ "$RUN_COMMON_CM_SCRIPT" == "true" ]; then
  RUN_DIR=$CMF_PACKAGE_DIR
else
  # Make sure that the custom script is executable
  chmod u+x $RUN_DIR/$CM_CSD_SCRIPT
fi

# Allow CSDs to use the common functions
export COMMON_SCRIPT=${cloudera_config}/cloudera-config.sh

exec $RUN_DIR/$CM_CSD_SCRIPT "$@"
