#!/bin/bash

# Copyright (c) 2018 Cloudera, Inc. All rights reserved.
#
# Purges log and metadata directories in .cm/distcp and .cm/hive-staging
# based on the modification time of the root directory.


current_dir=`dirname $0`
. ${current_dir}/distcp-base.sh

IFS=$'\n'
HADOOP_USER=hdfs
if [ "$1" == "-1" ]; then
  echo "disabled"
  exit 0
fi

expire_time=$(date -d "-$1 days" +%s)

IFS=' ' # change delimiter for splitting input
read -ra log_paths_to_expire <<< $2
IFS=$'\n' # reset delimiter

DELETED=()
DISTCP_LOG_PATH="/.cm/distcp"
HIVE_STAGING_PATH="/.cm/hive-staging"

delete_found_files () {
  FOUND=$(hadoop fs -ls $1 2> /dev/null)
  for log in $FOUND
  do
    found_date=$(echo $log | awk '{ print $6, $7}')
    found_dir=$(echo $log | awk '{ print $8}')

    if [ -z "$found_dir" ]; then
      continue
    fi

    echo "date:" $found_date
    echo "dir:" $found_dir
    mod_time=$(date -d $found_date +%s)

    if (( $mod_time < $expire_time )) ; then
      echo "Process " $found_dir
        DELETED+=(`hadoop fs -du -s $found_dir`)
        hadoop fs -rm -R -skipTrash $found_dir 2> /dev/null
    fi
  done
}

delete_user_logs () {
  USERS=$(hadoop fs -ls /user | awk '{ print $8 }')
  for DIR in $USERS
  do
    user=${DIR/\/user\//}
    DIR_DISTCP=$DIR$DISTCP_LOG_PATH
    delete_found_files $DIR_DISTCP

    DIR_HIVE_STAGING=$DIR$HIVE_STAGING_PATH
    delete_found_files $DIR_HIVE_STAGING
  done
}

delete_custom_logs () {
  for DIR in ${log_paths_to_expire[@]}
  do
    DIR_DISTCP=$DIR$DISTCP_LOG_PATH
    delete_found_files $DIR_DISTCP

    DIR_HIVE_STAGING=$DIR$HIVE_STAGING_PATH
    delete_found_files $DIR_HIVE_STAGING
  done
}

# delete logs and Hive replication metadata from /user directories
delete_user_logs

# delete logs and Hive replication metadata from custom set directories
delete_custom_logs

# store results of directories deleted
echo -e "logical\tphysical\tname" >  logs/results.txt
for each in "${DELETED[@]}"
do
  echo "$each" >> logs/results.txt
done