#!/bin/bash

# Copyright (c) 2011-2014 Cloudera, Inc. All rights reserved.

# Time marker for both stderr and stdout
date; date 1>&2

echo "Running on: $(hostname -f) ($(hostname -i))" 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# Load the parcel environment
source_parcel_environment

# attempt to find java
locate_cdh_java_home

echo "using $JAVA_HOME as JAVA_HOME"
echo "using $CDH_VERSION as CDH_VERSION"
echo "using $HBASE_HOME as HBASE_HOME"
echo "using $CONF_DIR as HBASE_CONF_DIR"
echo "using $CONF_DIR as HADOOP_CONF_DIR"
echo "using $HADOOP_HOME as HADOOP_HOME"

set_hbase_classpath

export HADOOP_HOME=$CDH_HADOOP_HOME
export HBASE_HOME=$CDH_HBASE_HOME
export HBASE_MASTER_OPTS=$(replace_pid $HBASE_MASTER_OPTS)
export HBASE_REGIONSERVER_OPTS=$(replace_pid $HBASE_REGIONSERVER_OPTS)
export HBASE_REST_OPTS=$(replace_pid $HBASE_REST_OPTS)
export HBASE_THRIFT_OPTS=$(replace_pid $HBASE_THRIFT_OPTS)
export HBASE_MASTER_OPTS=$(replace_pid $HBASE_MASTER_OPTS)

# debug
set -x

get_gc_args() {
  # HBase can supply JVM GC args here [ applicable for CDH >=6.3.0 ]

  # Uncomment below to override GC arguments provided by cloudera-config.sh:
  ##  JAVA8_GC_TUNING_ARGS="GC argument for JAVA8"
  ##  JAVA11_GC_TUNING_ARGS="GC argument for JAVA8"
  # function from cloudera-config.sh provides BASIC_GC_TUNING_ARGS based on java version
  set_basic_gc_tuning_args_based_on_java_version
}

# HBASE has been only tuning master, regionserver, rest and thrift roles with the
# same GC parameters.
get_gc_args
MASTER_GC_ARGS="$BASIC_GC_TUNING_ARGS"
REGIONSERVER_GC_ARGS="$BASIC_GC_TUNING_ARGS"
REST_GC_ARGS="$BASIC_GC_TUNING_ARGS"
THRIFT_GC_ARGS="$BASIC_GC_TUNING_ARGS"

# REGIONSERVER_GC_ARGS needs further GC arg changes
# fetch java version
get_java_major_version JAVA_MAJOR
ADDITIONAL_REGIONSERVER_GC_TUNING_ARGS=""
# populate BASIC_GC_TUNING_ARGS based on java version
case $JAVA_MAJOR in
  8)   ADDITIONAL_REGIONSERVER_GC_TUNING_ARGS="-verbose:gc -XX:+PrintGCDetails -XX:+PrintGCDateStamps"
       ;;
  11)  ADDITIONAL_REGIONSERVER_GC_TUNING_ARGS="-verbose:gc -Xlog:gc"
       ;;
  *)   echo "Unable to detect JAVA version. Skip using GC tuning args"
       ;;
esac
REGIONSERVER_GC_ARGS="$REGIONSERVER_GC_ARGS $ADDITIONAL_REGIONSERVER_GC_TUNING_ARGS"

# If safety valve ENV value is in-use, then user provided GC args are used
[[ ! -z $MASTER_JVM_GC_ARGS_SAFETY_VALVE ]] && MASTER_GC_ARGS="$MASTER_JVM_GC_ARGS_SAFETY_VALVE"
[[ ! -z $REGIONSERVER_JVM_GC_ARGS_SAFETY_VALVE ]] && REGIONSERVER_GC_ARGS="$REGIONSERVER_JVM_GC_ARGS_SAFETY_VALVE"
[[ ! -z $REST_JVM_GC_ARGS_SAFETY_VALVE ]] && REST_GC_ARGS="$REST_JVM_GC_ARGS_SAFETY_VALVE"
[[ ! -z $THRIFT_JVM_GC_ARGS_SAFETY_VALVE ]] && THRIFT_GC_ARGS="$THRIFT_JVM_GC_ARGS_SAFETY_VALVE"

# Now, replace the final GC args within the respective OPTs args
# NOTE: the below replacement only works if CDH >= 6.3.0 [ i.e the version (& future versions) that supports java 11 for CDH ]
export HBASE_MASTER_OPTS=$(replace_gc_args "$HBASE_MASTER_OPTS" "$MASTER_GC_ARGS")
# Only for HBASE_REGIONSERVER_OPTS
#
# move pattern '{{JAVA_GC_ARGS}}' to the end of the param list. This will ensure the options provided by user (from CM Web UI) take precedence over the ADDITIONAL_REGIONSERVER_GC_TUNING_ARGS and JAVA_GC_ARGS for HBASE_REGIONSERVER_OPTS
# For example, Instead of
# -Xms555745280 -Xmx555745280 -Xlog:gc+heap+age+region=trace:time,uptime {{JAVA_GC_ARGS}} -XX:ReservedCodeCacheSize=256m -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/tmp/HBASE-1_HBASE-1-REGIONSERVER-1a1076a5e5cf4e7668f7dfef082371e9_pid10290.hprof -XX:OnOutOfMemoryError=/opt/cloudera/cm-agent/service/common/killparent.sh
#
# the pattern JAVA_GC_ARGS will be moved to the end
# -Xms555745280 -Xmx555745280 -Xlog:gc+heap+age+region=trace:time,uptime  -XX:ReservedCodeCacheSize=256m -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/tmp/HBASE-1_HBASE-1-REGIONSERVER-1a1076a5e5cf4e7668f7dfef082371e9_pid10290.hprof -XX:OnOutOfMemoryError=/opt/cloudera/cm-agent/service/common/killparent.sh {{JAVA_GC_ARGS}}
#
HBASE_REGIONSERVER_OPTS=$(move_pattern_to_end "$HBASE_REGIONSERVER_OPTS")
#
# such that when JAVA_GC_ARGS is replaced by its true value. In this case, when JAVA_GC_ARGS is set to NULL by get_gc_args(), it will let '-Xlog:gc+heap+age+region=trace:time,uptime' to take precedence over '-verbose:gc -Xlog:gc' (that is the value from ADDITIONAL_REGIONSERVER_GC_TUNING_ARGS)
# thus the result for HBASE_REGIONSERVER_OPTS will be
# -Xms555745280 -Xmx555745280 -XX:ReservedCodeCacheSize=256m -Xlog:gc+heap+age+region=trace:time,uptime -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/tmp/HBASE-1_HBASE-1-REGIONSERVER-1a1076a5e5cf4e7668f7dfef082371e9_pid28709.hprof -XX:OnOutOfMemoryError=/opt/cloudera/cm-agent/service/common/killparent.sh -verbose:gc -Xlog:gc'
#
#
export HBASE_REGIONSERVER_OPTS=$(replace_gc_args "$HBASE_REGIONSERVER_OPTS" "$REGIONSERVER_GC_ARGS")
export HBASE_REST_OPTS=$(replace_gc_args "$HBASE_REST_OPTS" "$REST_GC_ARGS")
export HBASE_THRIFT_OPTS=$(replace_gc_args "$HBASE_THRIFT_OPTS" "$THRIFT_GC_ARGS")

# we set HADOOP_CONF_DIR explicitly to our CONF_DIR since otherwise the hbase
# script will use the default system hadoop configuration.
export HADOOP_CONF_DIR=$CONF_DIR

# OPSAPS-16739 - these 2 env vars are needed for HBase to pick its conf correctly
export HBASE_CONF_DIR=$CONF_DIR
export HADOOP_CONF=$CONF_DIR

# Search-replace {{CMF_CONF_DIR}} in files
replace_conf_dir

# Make sure topology.py and cloudera_manager_agent_fencer.py (where applicable) are executable
make_scripts_executable

acquire_kerberos_tgt hbase.keytab

# Disable IPv6.
export HBASE_OPTS="-Djava.net.preferIPv4Stack=true $HBASE_OPTS"

locate_hbase_script

function canary_echo() {
  echo "`date` RS pid:$$ " "$@"
}

function hbase_canary() {
  canary_echo "Sleeping before starting the canary"
  sleep 30
  if [ "$JAAS_FILE" != "" ]; then
    export HBASE_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_OPTS"
  fi
  export HBASE_LOGFILE=$CANARY_LOG_FILE
  while true;
  do
    canary_echo "Starting the canary"
    $HBASE_BIN --config $CONF_DIR org.apache.hadoop.hbase.tool.Canary -t $CANARY_TIMEOUT -daemon \
      -interval $CANARY_INTERVAL -regionserver ${RS_HOST}
    CANARY_EXIT=$?

    if [ $CANARY_EXIT -gt 2 ]; then
      canary_echo "Canary exited with error code $CANARY_EXIT"
      # Kill the process if user specified so
      if [ "$KILL_ON_CANARY_ERROR" = "true" ]; then
        # Sleep for a little bit just in case this is just the RS going down gracefully
        sleep 5
        # Try to terminate the process
        if kill -0 $$; then
          canary_echo "Stopping RegionServer gracefully"
          kill -15 $$
          sleep 10
        fi
        if kill -0 $$; then
          canary_echo "Killing the RegionServer process"
          kill -9 $$
        fi
        exit 1
      fi
    fi

    # Finally check that the monitored process is still running (a bit of belt'n'suspenders)
    if ! kill -0 $$ ; then
      canary_echo "Looks like the RegionServer exited. Canary runner thread is now exiting..."
      exit 0
    fi
  done
}

function znode_cleanup() {
  echo "`date` Starting znode cleanup thread with HBASE_ZNODE_FILE=$HBASE_ZNODE_FILE for $1"
  HBASE_OPTS=$(replace_pid $HBASE_OPTS)
  if [ "$JAAS_FILE" != "" ]; then
    export HBASE_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_OPTS"
  fi
  LOG_FILE=$CONF_DIR/logs/znode_cleanup.log
  
  # Check if parent PID is not running anymore
  set +x # Turn off debugging to avoid lots of logs in stderr
  while kill -0 $$; do sleep 1; done
  set -x

  # If znode file still exists then parent didn't exit gracefully
  RET=0
  if [ -f $HBASE_ZNODE_FILE ]; then
    echo "`date` Znode file exists. Will clean up the znode" >> $LOG_FILE
    if [ "$1" = "master" ]; then
      $HBASE_BIN --config $CONF_DIR master clear >> $LOG_FILE 2>&1
      RET=$?
    else
      #call ZK to delete the node
      ZNODE=`cat $HBASE_ZNODE_FILE`
      $HBASE_BIN --config $CONF_DIR zkcli delete $ZNODE >> $LOG_FILE 2>&1
      RET=$?
    fi
    if [ $RET -ne 0 ]; then
      echo "`date` Failed to clear znode" >> $LOG_FILE
    fi
    rm $HBASE_ZNODE_FILE
    exit $RET
  fi
  echo "`date` Znode file does not exist. No cleanup required." >> $LOG_FILE
  exit 0
}

if [ "upgrade" = "$1" ]; then
  # simulate /etc/default/hadoop if necessary
  . ${cloudera_config}/cdh-default-hadoop
  HDFS_BIN=$HADOOP_HDFS_HOME/bin/hdfs
  $HDFS_BIN --config $CONF_DIR dfs -test -d $HBASE_ROOTDIR/.cdh4-snapshot
  if [ $? -eq 0 ]; then
    echo "Renaming snapshot directory to .hbase-snapshot"
    $HDFS_BIN --config $CONF_DIR dfs -mv $HBASE_ROOTDIR/.cdh4-snapshot $HBASE_ROOTDIR/.hbase-snapshot
    if [ $? -ne 0 ]; then
      echo "Failed to rename snapshot directory"
      exit 1
    fi
  fi
fi

if [ "region_mover" = "$1" ]; then
  if [ $CDH_VERSION -lt 6 ]; then
    REGION_MOVER_SCRIPT="${cloudera_config}/../hbase/region_mover.cdh$CDH_VERSION.rb"
    echo "Using $REGION_MOVER_SCRIPT as REGION_MOVER_SCRIPT"
    if [ ! -e $REGION_MOVER_SCRIPT ]; then
      echo "Error: $REGION_MOVER_SCRIPT not found"
      exit 1
    fi
  fi
  EXTRA_ARGS=""
  if [ "unload" = "$2" ]; then
    EXTRA_ARGS="-x $CONF_DIR/excludes.txt"
  elif [ "load" != "$2" ]; then
    echo "Error: Unknown argument $2. Must be either load or unload"
    exit 1
  fi
  if [ -n "$REGION_MOVER_MAX_THREADS" ]; then
    EXTRA_ARGS="$EXTRA_ARGS -m $REGION_MOVER_MAX_THREADS"
  fi

  # $2 = 'load' for reload and 'unload' for decommission
  # $3 is hostname to reload/decommission.
  if [ $CDH_VERSION -lt 6 ]; then
    exec $HBASE_BIN --config $CONF_DIR org.jruby.Main $REGION_MOVER_SCRIPT $EXTRA_ARGS $2 $3
  else
    exec $HBASE_BIN --config $CONF_DIR org.apache.hadoop.hbase.util.RegionMover $EXTRA_ARGS -o $2 -r $3
  fi
elif [ "toggle_balancer" = "$1" ]; then
  # $2 tells whether to turn balancer on.
  SHELL_CMD="balance_switch $2"
  tmpfile=$(mktemp)
  echo $SHELL_CMD > $tmpfile
  echo exit >> $tmpfile
  $HBASE_BIN --config $CONF_DIR shell -n $tmpfile
  RET=$?
  rm $tmpfile
  exit $RET
elif [ "shell" = "$1" ]; then
  SHELL_CMD="${@:2}"
  tmpfile=$(mktemp)
  echo $SHELL_CMD > $tmpfile
  echo exit >> $tmpfile
  OUTPUT=$( $HBASE_BIN --config $CONF_DIR shell -n $tmpfile 2>&1 )
  RET=$?
  rm $tmpfile
  if [ $RET -ne 0 ]; then
    echo "Failed to $SHELL_CMD"
    exit $RET
  elif [[ "$SHELL_CMD" == *is_enabled* ]]; then
    if [[ "$OUTPUT" == *true* ]]; then
      cm_success_and_exit HBASE_TABLE_ENABLED
    fi
    cm_success_and_exit HBASE_TABLE_DISABLED
  fi
  exit 0
elif [ "remoteSnapshotTool" = "$1" ]; then
  if [ "export" = "$2" -o "import" = "$2" ]; then
    MR_CONF_DIR=${CONF_DIR}/mr-conf
    if [ "MAPREDUCE" = "$MAPREDUCE_SERVICE_TYPE" ]; then
      HADOOP_HOME=${CDH_MR1_HOME}
      echo "using $CDH_MR1_HOME as CDH_MR1_HOME"
      perl -pi -e "s#\{\{CDH_MR1_HOME}}#$CDH_MR1_HOME#g" $MR_CONF_DIR/*
    elif [ "YARN" = "$MAPREDUCE_SERVICE_TYPE" ]; then
      HADOOP_HOME=${CDH_HADOOP_HOME}
      echo "using $CDH_MR2_HOME as CDH_MR2_HOME"
      perl -pi -e "s#\{\{CDH_MR2_HOME}}#$CDH_MR2_HOME#g" $MR_CONF_DIR/*
    else
      echo "Invalid value for MAPREDUCE_SERVICE_TYPE: $MAPREDUCE_SERVICE_TYPE"
      exit 1
    fi
    TOOL_CONF_DIR=$MR_CONF_DIR
  else
    TOOL_CONF_DIR=$CONF_DIR
  fi

  REMOTE_SNAPSHOT_TOOL_JAR=$(ls ${MGMT_HOME}/lib/dr/hbase-remote-snapshot-utils-*.jar | head -n 1)
  HBASE_CLASSPATH="${HBASE_CLASSPATH}:${REMOTE_SNAPSHOT_TOOL_JAR}"
  TOOL_ARGS="${@:2}"
  exec $HBASE_BIN --config $TOOL_CONF_DIR com.cloudera.enterprise.bdr.snapshots.hbase.RemoteSnapshotTool $TOOL_ARGS
else
  # check if a jaas config file is provided for zookeeper authentication
  if [ "$JAAS_FILE" != "" ]; then
    echo "using $JAAS_FILE as JAAS_FILE"
    export HBASE_MANAGES_ZK=false
    if [ "master" = "$1" ]; then
      export HBASE_MASTER_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_MASTER_OPTS"
    elif [ "regionserver" = "$1" ]; then
      export HBASE_REGIONSERVER_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_REGIONSERVER_OPTS"
    elif [ "rest" = "$1" ]; then
      export HBASE_REST_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_REST_OPTS"
    elif [ "thrift" = "$1" ]; then
      export HBASE_THRIFT_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_THRIFT_OPTS"
    elif [ "upgrade" = "$1" ]; then
      export HBASE_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_MASTER_OPTS $HBASE_OPTS"
    fi
  fi
  if [ "regionserver" = "$1" -a -n "$CANARY_TIMEOUT" ]; then
    hbase_canary &
  fi
  if [ "start" = "$2" -a $CDH_VERSION -gt 4 ]; then
    if [ "regionserver" = "$1" -o "master" = "$1" ]; then
      # Use PID for the znode file in case process auto-restart is enabled
      export HBASE_ZNODE_FILE="$CONF_DIR/znode$$"
      znode_cleanup $1 &
    fi
  fi

  exec $HBASE_BIN --config $CONF_DIR "$@"
fi
