#!/bin/bash

# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# Load the parcel environment
source_parcel_environment

# attempt to find java
locate_cdh_java_home

# simulate /etc/default/hadoop
. ${cloudera_config}/cdh-default-hadoop

echo "Using $JAVA_HOME as JAVA_HOME"
echo "Using $CDH_VERSION as CDH_VERSION"
echo "Using $CONF_DIR as CONF_DIR"
echo "Using $CDH_HTTPFS_HOME as CDH_HTTPFS_HOME"

if [ $CDH_VERSION -gt 5 ]; then
  using_catalina="false"
else
  echo "Using $TOMCAT_HOME as TOMCAT_HOME"
  using_catalina="true"
fi

# mirror /etc/default/hadoop-httpfs

HTTPFS_RUN=$CONF_DIR/run/
export HTTPFS_USER=httpfs
export HTTPFS_CONFIG=$CONF_DIR
export HTTPFS_TEMP=$HTTPFS_RUN
set_hadoop_classpath

if [ "${using_catalina}" == "true" ]; then
  export HTTPFS_CATALINA_HOME=$TOMCAT_HOME
  export CATALINA_PID=${HTTPFS_RUN}hadoop-httpfs-httpfs.pid
  export CATALINA_TMPDIR=$HTTPFS_RUN

  if [ -z $SCM_DEFINES_SCRIPTS ]; then
    TOMCAT_CONF=/etc/hadoop-httpfs/$HTTPFS_SERVER_DIR
  else
    TOMCAT_CONF=$CDH_HTTPFS_HOME/../../etc/hadoop-httpfs/$HTTPFS_SERVER_DIR
  fi
  export CATALINA_BASE=/var/lib/hadoop-httpfs/tomcat-deployment

  env TOMCAT_CONF=${TOMCAT_CONF} TOMCAT_DEPLOYMENT=${CATALINA_BASE} HTTPFS_HOME=${CDH_HTTPFS_HOME} \
    bash ${CDH_HTTPFS_HOME}/tomcat-deployment.sh
else
  export HTTPFS_HOME=${CDH_HTTPFS_HOME}
fi

# log related env vars for our log4j.properties
#
# And disable IPv6.
#
# SCM is expected to set the following:
# - HTTPFS_LOG_DIR
# - HTTPFS_LOGFILE
# - HTTPFS_ROOT_LOGGER
# - HTTPFS_HTTP_PORT (pass thru)
# - HTTPFS_ADMIN_PORT (pass thru)
export HTTPFS_LOG=$HTTPFS_LOG_DIR
OPTIONS="$OPTIONS -Dhadoop.log.file=${HTTPFS_LOGFILE}"
OPTIONS="$OPTIONS -Dhadoop.root.logger=${HTTPFS_ROOT_LOGGER}"
OPTIONS="$OPTIONS -Djava.net.preferIPv4Stack=true"

# debug
set -x

# Search-replace {{CMF_CONF_DIR}} in files
replace_conf_dir

if [ "${using_catalina}" == "true" ]; then
  CATALINA_OPTS=$(replace_pid $CATALINA_OPTS)
  export CATALINA_OPTS="$OPTIONS $CATALINA_OPTS"

  exec $CDH_HTTPFS_HOME/sbin/httpfs.sh run
else
  export HADOOP_OPTS="$OPTIONS $HADOOP_OPTS"
  export HADOOP_LOG_DIR=$HTTPFS_LOG_DIR

  exec $CDH_HDFS_HOME/bin/hdfs httpfs
fi
