#!/bin/bash

# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# Load the parcel environment
source_parcel_environment

# Attempt to find Java and place it in JAVA_HOME. The catalina.sh script expects
# to find it there.
locate_cdh_java_home

set -x

echo "using $CDH_VERSION as CDH_VERSION"

# Search-replace {{CMF_CONF_DIR}} in files
replace_conf_dir

get_gc_args() {
  # Keystoreindexer can supply JVM GC args here [ applicable for CDH >=6.3.0 ]

  # Uncomment below to override GC arguments provided by cloudera-config.sh:
  ##  JAVA8_GC_TUNING_ARGS="GC argument for JAVA8"
  ##  JAVA11_GC_TUNING_ARGS="GC argument for JAVA8"
  # function from cloudera-config.sh provides BASIC_GC_TUNING_ARGS based on java version
  set_basic_gc_tuning_args_based_on_java_version
}

if [ "migrate-policy-file" = "$1" ]; then
  POLICY_FILE=$2
  SERVICE_NAME=$3
  SENTRY_CONF=$CONF_DIR/$4
  exec $CDH_HBASE_INDEXER_HOME/cloudera/hbase-indexer-sentry -mgr -i -f "$POLICY_FILE" -s "$SERVICE_NAME" --sentry_conf "$SENTRY_CONF"
else
  # invoke hbase-indexer
  HBASE_INDEXER_OPTS=$(replace_pid $HBASE_INDEXER_OPTS)

  # HBASE has been only tuning the hbaseindexer role
  get_gc_args
  HBASE_INDEXER_GC_ARGS="$BASIC_GC_TUNING_ARGS"

  # If safety valve ENV value is in-use, then user provided GC args are used
  [[ ! -z $HBASE_INDEXER_JVM_GC_ARGS_SAFETY_VALVE ]] && HBASE_INDEXER_GC_ARGS="$HBASE_INDEXER_JVM_GC_ARGS_SAFETY_VALVE"

  # Now, replace the final GC args within the respective OPTs args
  # NOTE: the below replacement only works if CDH >= 6.3.0 [ i.e the version (& future versions) that supports java 11 for CDH ]
  export HBASE_INDEXER_OPTS=$(replace_gc_args "$HBASE_INDEXER_OPTS" "$HBASE_INDEXER_GC_ARGS")

  export HBASE_INDEXER_OPTS="-Djava.net.preferIPv4Stack=true $HBASE_INDEXER_OPTS"
  if [ "$JAAS_FILE" != "" ]; then
    export HBASE_INDEXER_OPTS="-Djava.security.auth.login.config=$CONF_DIR/$JAAS_FILE $HBASE_INDEXER_OPTS"
  fi
  export HBASE_INDEXER_CONF_DIR=$CONF_DIR
  exec $CDH_HBASE_INDEXER_HOME/bin/hbase-indexer "$@"
fi
