#!/bin/bash

# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# Load the parcel environment
source_parcel_environment

# Attempt to find Java and place it in JAVA_HOME. The catalina.sh script expects
# to find it there.
locate_cdh_java_home

set -x

echo "using $CDH_VERSION as CDH_VERSION"

# This horrifying mess of variables represents an amalgam of the actions
# performed by oozie-run.sh and other Oozie helper scripts. We can't use these
# scripts directly because SCM disagrees with too many of their assumptions
# (e.g. the Oozie configuration is in /etc).
#
# SCM is expected to set the following:
# - oozie_log_dir
# - oozie_log_file
# - oozie_http_hostname
# - oozie_http_port
# - oozie_admin_port (CDH4 only)
# - oozie_web_console
# - OOZIE_SERVER_OPTS
# - OOZIE_SERVER_DIR (CDH4 only)
# - OOZIE_HTTPS_PORT (CDH4 + SSL only)
# - OOZIE_HTTPS_KEYSTORE_FILE (CDH4 + SSL only)
# - OOZIE_HTTPS_KEYSTORE_PASSWORD CDH4 + SSL only)
# - OOZIE_HTTPS_TRUSTSTORE_FILE (CDH4 + SSL only)
# - OOZIE_HTTPS_TRUSTSTORE_PASSWORD (CDH4 + SSL only)
# - OOZIE_HTTPS_CIPHERS (CDH5 + SSL only)

export OOZIE_HOME=$CDH_OOZIE_HOME
export OOZIE_CONFIG=$PWD
export OOZIE_LOG=$oozie_log_dir
export OOZIE_LOG_FILE=$oozie_log_file
export OOZIE_CONFIG_FILE=oozie-site.xml
export OOZIE_LOG4J_FILE=log4j.properties
export OOZIE_LOG4J_RELOAD=10
export OOZIE_HTTP_HOSTNAME=$oozie_http_hostname
export OOZIE_HTTP_PORT=$oozie_http_port
# For CDH5.x OOZIE_HTTPS_PORT is set in the environment. From C6 it was changed
# to oozie_https_port. The old value shoul not be overwritten.
if [ -z "${OOZIE_HTTPS_PORT}"]; then
  export OOZIE_HTTPS_PORT=$oozie_https_port
fi

USES_TOMCAT=0
START_SERVER=0
if [ "install-sharelib" != "$1" -a "db-command" != "$1" ]; then
  # Oozie server as of CDH6 is jetty based.
  START_SERVER=1
  if [ $CDH_VERSION -lt 6 ]; then
    USES_TOMCAT=1
  fi
fi

if [ "${USES_TOMCAT}" = 1 ]; then
  export OOZIE_ADMIN_PORT=$oozie_admin_port
  if [ $CDH_VERSION -ge 5 ]; then
    if [ -z "$SCM_DEFINES_SCRIPTS" ]; then
      TOMCAT_CONF=/etc/oozie/$OOZIE_SERVER_DIR
    else
      TOMCAT_CONF=$CDH_OOZIE_HOME/../../etc/oozie/$OOZIE_SERVER_DIR
    fi
    export CATALINA_BASE=/var/lib/oozie/tomcat-deployment

    env TOMCAT_CONF=${TOMCAT_CONF} TOMCAT_DEPLOYMENT=${CATALINA_BASE} OOZIE_HOME=${CDH_OOZIE_HOME} \
      bash ${CDH_OOZIE_HOME}/tomcat-deployment.sh
  else
    export CATALINA_BASE=$OOZIE_HOME/$OOZIE_SERVER_DIR
  fi

  echo "using $CATALINA_BASE as CATALINA_BASE"
  if [ ! -d "${CATALINA_BASE}" ]; then
    echo "$CATALINA_BASE doesn't exist"
    exit 1
  fi

  export CATALINA_TMPDIR=$PWD/temp
  export CATALINA_OUT=$OOZIE_LOG
  export CATALINA_PID=$PWD/oozie.pid

  mkdir -p $CATALINA_TMPDIR
fi

# Search-replace {{CMF_CONF_DIR}} in files
replace_conf_dir

# local replacement of substitution variables.
# i.e. master doesn't know final resting place of configuration files
#   which may require absolute path (e.g. dfs_allows)
if [ -d "$CONF_DIR/yarn-conf" ]; then
  export HADOOP_CONF_DIR=$CONF_DIR/yarn-conf
  export HADOOP_MAPRED_HOME=$CDH_MR2_HOME
  perl -pi -e "s#\{\{CDH_MR2_HOME}}#$CDH_MR2_HOME#g" $HADOOP_CONF_DIR/*
  perl -pi -e "s#\{\{CDH_HADOOP_HOME}}#$CDH_HADOOP_HOME#g" $HADOOP_CONF_DIR/*
else
  export HADOOP_CONF_DIR=$CONF_DIR/hadoop-conf
  export HADOOP_MAPRED_HOME=$CDH_MR1_HOME
  perl -pi -e "s#\{\{CDH_MR1_HOME}}#$CDH_MR1_HOME#g" $HADOOP_CONF_DIR/*
  perl -pi -e "s#\{\{CDH_HADOOP_HOME}}#$CDH_HADOOP_HOME#g" $HADOOP_CONF_DIR/*
fi


OOZIE_SERVER_OPTS=$(replace_pid $OOZIE_SERVER_OPTS)
OPTIONS="$OOZIE_SERVER_OPTS"
OPTIONS="$OPTIONS -Doozie.home.dir=${OOZIE_HOME}"
OPTIONS="$OPTIONS -Doozie.config.dir=$OOZIE_CONFIG"
OPTIONS="$OPTIONS -Doozie.log.dir=$OOZIE_LOG"
OPTIONS="$OPTIONS -Doozie.log.file=$OOZIE_LOG_FILE"
OPTIONS="$OPTIONS -Doozie.config.file=$OOZIE_CONFIG_FILE"
OPTIONS="$OPTIONS -Doozie.log4j.file=$OOZIE_LOG4J_FILE"
OPTIONS="$OPTIONS -Doozie.log4j.reload=$OOZIE_LOG4J_RELOAD"

if [ "${USES_TOMCAT}" = 1 -o "${START_SERVER}" = 0 ]; then
  OPTIONS="$OPTIONS -Doozie.http.hostname=$OOZIE_HTTP_HOSTNAME"
  OPTIONS="$OPTIONS -Doozie.http.port=$OOZIE_HTTP_PORT"
  OPTIONS="$OPTIONS -Djava.net.preferIPv4Stack=true" # Disable IPv6.
  OPTIONS="$OPTIONS -Doozie.admin.port=$OOZIE_ADMIN_PORT"
  OPTIONS="$OPTIONS -Dderby.stream.error.file=$OOZIE_LOG/derby.log"

  if [ $CDH_VERSION -ge 5 ]; then
    OPTIONS="$OPTIONS -Doozie.instance.id=$OOZIE_HTTP_HOSTNAME"
    if [ -e $CDH_HADOOP_HOME/lib/native/libhadoop.so ]; then
      OPTIONS="$OPTIONS -Djava.library.path=$CDH_HADOOP_HOME/lib/native"
    fi
  fi
fi

# Add truststore filename and truststore password to named variable
#
# Note, "set +x" is always done, caller must explicitly toggle off (-x)
# if so desired.
#
# param $1 - The name of the variable to append to. The function
# appends truststore properties to this variable.
function add_truststore_properties() {
  if [ -z $1 ]; then
    echo "Must call with the name of variable to assign."
    exit 1
  fi

  set +x
  if [ -n "$OOZIE_HTTPS_TRUSTSTORE_FILE" ]; then
    if [ ! -e "$OOZIE_HTTPS_TRUSTSTORE_FILE" ]; then
      echo "Trust store file $OOZIE_HTTPS_TRUSTSTORE_FILE does not exist"
      exit 1
    fi
    TRUSTSTORE_PROPERTIES="-Djavax.net.ssl.trustStore=$OOZIE_HTTPS_TRUSTSTORE_FILE"
    if [ -n "$OOZIE_HTTPS_TRUSTSTORE_PASSWORD" ]; then
      TRUSTSTORE_PROPERTIES="${TRUSTSTORE_PROPERTIES} -Djavax.net.ssl.trustStorePassword=$OOZIE_HTTPS_TRUSTSTORE_PASSWORD"
    fi
    eval OLD_VALUE="\$$1"
    NEW_VALUE="${OLD_VALUE} ${TRUSTSTORE_PROPERTIES}"
    eval $1="'${NEW_VALUE}'"
  fi
}

# Add options for SSL if it's set. Don't echo to stderr.
if [ "${USES_TOMCAT}" = 1 ]; then
  set +x
  if [ -n "$OOZIE_HTTPS_KEYSTORE_FILE" ]; then
    if [ ! -e "$OOZIE_HTTPS_KEYSTORE_FILE" ]; then
      echo "Key store file $OOZIE_HTTPS_KEYSTORE_FILE does not exist"
      exit 1
    fi
    if [ $CDH_VERSION -ge 5 ]; then
      # These go directly into the catalina.properties file in order to keep
      # them off of the command line.
      tomcat_set_prop "oozie.https.keystore.file=${OOZIE_HTTPS_KEYSTORE_FILE}"
      tomcat_set_prop "oozie.https.keystore.pass=${OOZIE_HTTPS_KEYSTORE_PASSWORD}"
    else
      OPTIONS="$OPTIONS -Doozie.https.keystore.file=${OOZIE_HTTPS_KEYSTORE_FILE}"
      OPTIONS="$OPTIONS -Doozie.https.keystore.pass=${OOZIE_HTTPS_KEYSTORE_PASSWORD}"
    fi
    OPTIONS="$OPTIONS -Doozie.https.port=$OOZIE_HTTPS_PORT"
  fi

  if [ -n "$OOZIE_HTTPS_TRUSTSTORE_FILE" ]; then
    add_truststore_properties OPTIONS
  fi

  OPTIONS="$OPTIONS -Doozie.https.ciphers=$OOZIE_HTTPS_CIPHERS"

  set -x
fi

# Copy JDBC jar if present
if [ "$OOZIE_JDBC_JAR" != "" ]; then
  eval JAR_LOCATION=\$$OOZIE_JDBC_JAR
  echo "Copying JDBC jar from $JAR_LOCATION to /var/lib/oozie"
  cp $JAR_LOCATION /var/lib/oozie/
fi

OOZIECPPATH=""
if [ "${USES_TOMCAT}" = 0 ]; then
  if [ -d "${OOZIE_HOME}/libtools" ]; then
    for i in "${OOZIE_HOME}/libtools/"*.jar; do
      OOZIECPPATH="${OOZIECPPATH}:$i"
    done
  fi
  if [ -d "${OOZIE_HOME}/libext" ]; then
    for i in "${OOZIE_HOME}/libext/"*.jar; do
      OOZIECPPATH="${OOZIECPPATH}:$i"
    done
  fi
fi

# DB Commands exec OozieDBCLI
if [ "db-command" = "$1" ]; then
  DBCOMMAND=$2
  if [ "" = "$DBCOMMAND" ]; then
    echo "db-command needs an additional argument."
    exit 1
  fi

  if [ "dump_db" = "$DBCOMMAND" -o "load_db" = "$DBCOMMAND" ]; then
    DUMP=$3
    if [ "" = "$DUMP" ]; then
       echo "$DBCOMMAND needs the location of database dump."
       exit 1
    fi

    if [ "dump_db" = "$DBCOMMAND" ]; then
       exec ${JAVA_HOME}/bin/java ${OPTIONS} -cp ${OOZIECPPATH} org.apache.oozie.tools.OozieDBExportCLI export "${DUMP}"
    fi

    if [ "load_db" = "$DBCOMMAND" ]; then
       exec ${JAVA_HOME}/bin/java ${OPTIONS} -cp ${OOZIECPPATH} org.apache.oozie.tools.OozieDBImportCLI import "${DUMP}"
    fi
  fi
  exec $JAVA_HOME/bin/java ${OPTIONS} -cp ${OOZIECPPATH} org.apache.oozie.tools.OozieDBCLI $DBCOMMAND -run
fi

# Install Sharelib execs OozieSharelibCLI
if [ "install-sharelib" = "$1" ]; then
  acquire_kerberos_tgt oozie.keytab
  if [ $CDH_VERSION -lt 5 ]; then
    # Expected args - $2 = sharelib file name, $3 = hadoop client config dir
    HDFS_CFG=$OOZIE_CONFIG/$3
    $CDH_HADOOP_BIN --config $HDFS_CFG fs -rmr $OOZIE_DIR/share

    # Unpack the sharelibs into the staging directory.
    UNPACKED_DIR=$(mktemp -d -t oozie-sharelib-staging.XXXXXXXXXX)
    tar -C $UNPACKED_DIR -xzf $OOZIE_HOME/$2 #$2 is the name of the tar file to put in HDFS
    $CDH_HADOOP_BIN --config $HDFS_CFG fs -put $UNPACKED_DIR/share $OOZIE_DIR/share
  else
    if [ -n "$6" ]; then
      # $2 is the name of the directory or tarball containing sharelib and $3 is fs.defaultFS for HDFS
      # $6 contains the concurrency level
      $JAVA_HOME/bin/java ${OPTIONS} -cp ${OOZIECPPATH} org.apache.oozie.tools.OozieSharelibCLI create -fs $3 -locallib $OOZIE_HOME/$2 -concurrency $6
    else
      $JAVA_HOME/bin/java ${OPTIONS} -cp ${OOZIECPPATH} org.apache.oozie.tools.OozieSharelibCLI create -fs $3 -locallib $OOZIE_HOME/$2
    fi
  fi
  UPLOAD_SUCCESS=$?
  if [ $UPLOAD_SUCCESS -ne 0 ]; then
    exit $UPLOAD_SUCCESS
  fi
  # Changing the permissions for the shared lib (OPSAPS-32451)
  if [ -n "$4" -a -n "$5" ]; then
    # Do not fail the whole command even if chmod fails (OPSAPS-35822)
    $CDH_HADOOP_BIN fs -chmod -R $4 $5 || echo "Ignoring failure in chmod"
  fi

  # Do we need to tell oozie server to reload shared library?
  if [ "true" = "${UPDATE_SHARELIB}" ]; then
    # Check that OOZIE_URL was set
    if [ -z "${OOZIE_URL}" ]; then
      echo "OOZIE_URL not defined"
      exit 1
    fi

    # Set truststore file and password if ssl
    if [ -n "$OOZIE_HTTPS_TRUSTSTORE_FILE" ]; then
      add_truststore_properties OOZIE_CLIENT_OPTS
      export OOZIE_CLIENT_OPTS
    fi

    # Tell running oozie to update shared lib
    exec ${OOZIE_HOME}/bin/oozie admin -sharelibupdate
  fi

  exit 0
fi

# The Oozie server case: start Tomcat if pre CDH6, start Jetty otherwise.
if [ "${USES_TOMCAT}" = 0 ]; then
  JETTY_CP=
  JETTY_CP="${OOZIE_HOME}/embedded-oozie-server/*"
  JETTY_CP="${JETTY_CP}:${OOZIE_HOME}/embedded-oozie-server/dependency/*"
  JETTY_CP="${JETTY_CP}:${OOZIE_HOME}/lib/*"
  JETTY_CP="${JETTY_CP}:${OOZIE_HOME}/libtools/*"
  JETTY_CP="${JETTY_CP}:${OOZIE_HOME}/embedded-oozie-server/webapp/WEB-INF/lib/*"
  JETTY_CP="${JETTY_CP}:/var/lib/oozie/*"
  JETTY_CP="${JETTY_CP}:${OOZIE_HOME}/embedded-oozie-server/"
  exec ${JAVA_HOME}/bin/java ${OPTIONS} \
      -Doozie.data.dir=/tmp/data \
      -Djava.library.path= \
      -Doozie.instance.id=${OOZIE_HTTP_HOSTNAME} \
      -Doozie.http.hostname=${OOZIE_HTTP_HOSTNAME} \
      -Doozie.http.port=${OOZIE_HTTP_PORT} \
      -Doozie.https.port=${OOZIE_HTTPS_PORT} \
      -cp ${JETTY_CP} \
      org.apache.oozie.server.EmbeddedOozieServer 2>&1
else
  export CATALINA_OPTS="$OPTIONS $CATALINA_OPTS"
  exec $TOMCAT_HOME/bin/catalina.sh run
fi
