#!/usr/bin/env python
# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

import os
import sys
import zipfile

import cmf.redaction

def archive_directory(dir_path, archive_path):
    """Archives the directory at the given dir_path to a zip archive at the 
    given archive_path. The output at archive_path will be a zip archive with
    the directory at dir_path at the top level.

    e.g. if dir_path is /home/me/foo and archive_path is /home/me/foo.zip, foo.zip
    will be a zip archive containing foo at the top level"""

    zip_file = zipfile.ZipFile(archive_path, mode="w", compression=zipfile.ZIP_DEFLATED)

    dir_path_dirname = os.path.dirname(dir_path)

    def add_files_to_archive(arg, dirname, filenames):
        for filename in filenames:
            file_path = os.path.join(dirname, filename)

            # Strip everything up to the directory being archived. We do this
            # so that os.path.basename(dir_path) resides at the top level of
            # the zip file.
            archive_name = file_path[len(dir_path_dirname) + 1:]
            if os.path.isdir(file_path):
                continue
            zip_file.write(file_path, archive_name)
            
    os.path.walk(dir_path, add_files_to_archive, None)
    zip_file.close()

if __name__ == "__main__":
    # Need to add clusterstats configuration to the path so that machine_stats 
    # initializes itself properly.
    sys.path.append(os.environ["CONF_DIR"])
    from clusterstats.client import machine_stats

    # HOST_STATISTICS_DIR is set by the server and sent to the agent.
    OUTPUT_DIR = os.path.join(os.environ["CONF_DIR"], os.environ["HOST_STATISTICS_DIR"])

    os.mkdir(OUTPUT_DIR)

    # Default timeout is 1 minute - each OS stat command must complete w/in a minute or be killed
    timeout = None

    if "TIMEOUT" in os.environ:
        timeout = os.environ["TIMEOUT"]

    if "REDACTION_RULES_FILE" in os.environ:
        redaction_rules_file = os.environ["REDACTION_RULES_FILE"]
        redactor = cmf.redaction.REDACTOR
        redactor.add_policy_from_file(redaction_rules_file)
        machine_stats.collect_sys_stats(OUTPUT_DIR, [redactor.redact], timeout=timeout)
    else:
        machine_stats.collect_sys_stats(OUTPUT_DIR, timeout=timeout)

    # Write zip file to CONF_DIR.
    archive_path = os.path.join(os.environ["CONF_DIR"], os.environ["HOST_STATISTICS_DIR"] + ".zip")
    archive_directory(OUTPUT_DIR, archive_path)
